@@ -32,6 +32,9 @@ lexing::classifier<char, TOKEN> build_classifier() {
     auto WS = just(' ') | just('\t');
     cl.insert(WS, TOKEN::TOK_WHITESPACE);
 
+    cl.insert(word(INLINE_OPEN_STR), TOKEN::TOK_OINLINE);
+    cl.insert(word(INLINE_CLOSE_STR), TOKEN::TOK_CINLINE);
+
     cl.insert(just('%'), TOKEN::TOK_PERCENT);
     cl.insert(just('@'), TOKEN::TOK_ATSYMBOL);
     cl.insert(just('*'), TOKEN::TOK_ASTERISK);
@@ -155,6 +158,7 @@ lexing::classifier<char, TOKEN> build_classifier() {
     cl.insert(iword("cell"), TOKEN::TOK_CELL);
     cl.insert(iword("text"), TOKEN::TOK_TEXT);
     cl.insert(iword("shape"), TOKEN::TOK_SHAPE);
+    cl.insert(iword("chart"), TOKEN::TOK_CHART);
     cl.insert(iword("space"), TOKEN::TOK_SPACE_COMMAND);
     cl.insert(iword("break"), TOKEN::TOK_BREAK_COMMAND);
     cl.insert(iword("multi"), TOKEN::TOK_MULTI_COMMAND);
@@ -245,6 +249,7 @@ TOKEN get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, READER_FN readandclassi
         case TOKEN::TOK_TILDE:
         case TOKEN::TOK_OPARENTHESIS:
         case TOKEN::TOK_CPARENTHESIS:
+        case TOKEN::TOK_CINLINE:
         case TOKEN::TOK_UNRECOGNIZED_CHAR:
         case TOKEN::TOK_EOF:
             loc = inp.template commit<false, false>(len);
@@ -370,6 +375,7 @@ TOKEN get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, READER_FN readandclassi
         case TOKEN::TOK_CLONE_MODIFIER_REPLACE:
         case TOKEN::TOK_CLONE_MODIFIER_BEFORE:
         case TOKEN::TOK_CLONE_MODIFIER_RECURSIVE:
+        case TOKEN::TOK_CHART:
 
         //special tokens, whose value is their text
         case TOKEN::TOK_NUMBER:
@@ -428,7 +434,7 @@ TOKEN get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, READER_FN readandclassi
             //scan "xxx" as TOK_XXX and not as TOK_STRING
             if (text.ends_with("..")) {
                 text.remove_suffix(2);
-                sv_reader<CSH> input(text, 0);
+                sv_reader<CSH> input(text, {});
                 std::tie(int_token, len) = readandclassify(0, input);
                 goto re_scanned;
             }
@@ -453,6 +459,43 @@ TOKEN get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, READER_FN readandclassi
             val.str.set(text);
             return TOKEN::TOK_QSTRING;
         }
+        case TOKEN::TOK_OINLINE: {
+            const auto loc_oinline = inp.template commit<false, false>(len);
+            loc = inp.template commit<false, false>(0);
+            const char* start_text_ptr = inp.view(0).data(), * end_text_ptr = start_text_ptr;
+            unsigned nesting_level = 0;
+            while (inp.has(1)) {
+                auto [int_token, len] = readandclassify(0, inp);
+                switch (TOKEN(int_token)) {
+                case TOKEN::TOK_OINLINE:
+                    nesting_level++;
+                    FALLTHROUGH;
+                default:
+                    if (!len) len = 1;
+                    end_text_ptr = inp.view(len).data() + len;
+                    loc.merge(inp.template commit<true, false>(len));
+                    break;
+                case TOKEN::TOK_CINLINE:
+                    if (nesting_level) {
+                        nesting_level--;
+                        end_text_ptr = inp.view(len).data() + len;
+                        loc.merge(inp.template commit<true, false>(len));
+                        break;
+                    }
+                    inp.template commit<true, false>(len);
+                    val.str = std::string_view(start_text_ptr, end_text_ptr - start_text_ptr);
+                    return TOKEN::TOK_INLINE;
+                }
+            }
+            //Unpaired OINLINE
+            if constexpr (CSH) {
+                C.AddCSH_Error(loc_oinline, "Unpaired beginning of inlined chart '" INLINE_OPEN_STR "'.");
+                C.AddCSH(loc, COLOR_COMMENT); //we have consumed the whole file, color as comment
+            } else
+                C.Error.Error(loc_oinline.start(), "Unpaired beginning of inlined chart '" INLINE_OPEN_STR "'.");
+            loc = loc_oinline;
+            return TOKEN::TOK_UNMATCHED_INLINE;
+        }
         default:
             _ASSERT(0);
             loc = inp.template commit<false, false>(len);
