@@ -31,7 +31,8 @@
 using namespace block;
 
 BlockCsh::BlockCsh(Csh::FileListProc proc, const LanguageCollection* languages) :
-    Csh(BlockContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, languages)
+    Csh(BlockContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, languages),
+    hadProcReplay(false)
 {
     ForbiddenStyles.erase("row");
     ForbiddenStyles.erase("col");
@@ -1130,7 +1131,7 @@ std::string BlockCsh::ReplaceEntityName(std::string_view full_entity_name, int b
     const size_t replace_len_in_full_name = full_entity_name.length()-replace_pos_in_full_name;
     //Walk the input UTF-8 text to find character positions
     for (const auto &[pos, fname] : poses) {
-        std::string_view src_name = input_text.substr(pos.first_pos.byte_index-first_pos.byte_index, pos.byte_length());
+        std::string_view src_name = std::string_view(input_text).substr(pos.first_pos.byte_index-first_pos.byte_index, pos.byte_length());
         //Locate what part of the text to replace: block name in source (src_name) may be a partial one
         //That is, if we replace A.B.C to D (becoming A.C.D in the process),
         //We may have in the source file (src_name may be):
