@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2023 Zoltan Turanyi
+	Copyright (C) 2008-2024 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -386,7 +386,7 @@ public:
     /** Returns the state of the Canvas*/
     bool TurnPage(const PBDataVector *pageBreakData, unsigned next_page,
                   MscError *pError=nullptr);
-    bool ErrorAfterCreation(MscError *pError,  const PBDataVector *pageBreakData);
+    bool ErrorAfterCreation(MscError* pError = nullptr, const PBDataVector* pageBreakData = nullptr);
 #ifdef CAIRO_HAS_FC_FONT
 protected:
     FcPattern* fc_pattern = nullptr; //Destroy this pattern in CloseOutput()
@@ -472,6 +472,7 @@ public:
     void ClipInverse(const Contour &area);
     /** Undo a previous clip operation*/
     void UnClip() { if (cannot_draw()) return; cairo_restore(cr);}
+    [[nodiscard]] bool Transform(const TRMatrix &);
     void Transform_Rotate(const XY &center, double radian);
     void Transform_Rotate(const XY &center, double cos, double sin);
     void Transform_Rotate90(double s, double d, bool clockwise);
