@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 #define ATTRIBUTE_H
 
 #include <string>
-#include <sstream>
+#include <utility>
 #include <list>
 #include <algorithm>
 #include <charconv>
@@ -79,6 +79,8 @@ template <class T> requires std::is_arithmetic_v<T>
 string&& operator <<(string&& s, T t) { return std::move(s.append(std::to_string(t))); }
 string& operator <<(string& s, bool t) = delete;
 string&& operator <<(string&& s, bool t) = delete;
+inline string& operator <<(string& s, char c) { s.push_back(c); return s; };
+inline string&& operator <<(string&& s, char c) { s.push_back(c); return std::move(s); }
 inline string& operator <<(string& s, const char *t) { return s.append(t); } //needed or it converts to bool
 inline string&& operator <<(string&& s, const char* t) { return std::move(s.append(t)); }
 inline string &operator <<(string&s, std::string_view t) { return s.append(t); }
@@ -101,6 +103,8 @@ int CaseInsensitiveBeginsWith(std::string_view a, std::string_view b) noexcept;
 inline bool CaseInsensitiveEqual(std::string_view a, std::string_view b) noexcept
     {return a.length()==b.length() && CaseInsensitiveBeginsWith(a,b)==2;}
 
+bool CaseInsensitiveSmaller(std::string_view a, std::string_view b) noexcept;
+
 /** Checks how an attribute name ends (dot-sensitive).
  * Note that attribute names can be segmented with dots. This function
  * checks if one or more of the last segments are equal to `a`.
@@ -381,27 +385,20 @@ bool Convert(std::string_view name, Enum &a)
 };
 
 /** Returns a slash ('/') separated list of possible values for the type of `dummy1`.*/
-template <typename Enum>
-string CandidatesFor(Enum)
-{
-    string s(EnumEncapsulator<Enum>::names[1]);
-    for (int i=2; EnumEncapsulator<Enum>::names[i][0]; i++){
-        s.append("/");
-        s.append(EnumEncapsulator<Enum>::names[i]);
-    }
+template <typename Enum> requires std::is_enum_v<Enum>
+string CandidatesFor(const char *separator = "/", Enum from = Enum(1),  Enum till = Enum(INT_MAX)) {
+    std::string s;    
+    for (std::underlying_type_t<Enum> i = 1; EnumEncapsulator<Enum>::names[i][0]; i++)
+        if (std::to_underlying(from) <= i && i <= std::to_underlying(till)) {
+            if (s.size()) s.append(separator);
+            s.append(EnumEncapsulator<Enum>::names[i]);
+        }
     return s;
 };
 
 /** Returns a slash ('/') separated list of possible values for the type of `dummy1`.*/
 template <typename Enum>
-string CandidatesFor() {
-    string s(EnumEncapsulator<Enum>::names[1]);
-    for (int i = 2; EnumEncapsulator<Enum>::names[i][0]; i++) {
-        s.append("/");
-        s.append(EnumEncapsulator<Enum>::names[i]);
-    }
-    return s;
-};
+string CandidatesFor(Enum, const char *separator = "/", Enum from = Enum(1), Enum till = Enum(INT_MAX)) { return CandidatesFor<Enum>(separator, from, till); }
 
 ////////////////////////////////////////////////////////////
 
