@@ -22,7 +22,7 @@
 #define _CRT_SECURE_NO_DEPRECATE //For visual studio to allow sscanf
 #include "cgen_shapes.h"
 
-const char ShapeElement::act_code[] = "LMCEHTSSSP";
+constexpr const char ShapeElement::act_code[];
 
 string ShapeElement::ErrorMsg(Type t, int numargs)
 {
@@ -38,7 +38,8 @@ string ShapeElement::ErrorMsg(Type t, int numargs)
     case TEXT_AREA: ret = "T (Text area) commands require four numbers, two 2D points to represent a rectangle."; break;
     case SECTION_BG:
     case SECTION_FG_LINE:
-    case SECTION_FG_FILL: ret = "S (section) commands require one number: the section number [0-2]"; break;
+    case SECTION_FG_FILL:
+    case SECTION_FG_BG: ret = "S (section) commands require one number: the section number [0-3]"; break;
     case PORT: ret = "P (Port) commands require two numbers, an X and an Y coordinate and the name of the port (case sensitive and alphanumeric) and optionally its direction (0-360 degrees from north clockwise; -1: no specific dir; -2:perpendicular."; break;
     }
     return ret;
@@ -48,7 +49,7 @@ string ShapeElement::ErrorMsg(Type t, int numargs)
 string ShapeElement::Write() const
 {
     string ret(1, act_code[action]);
-    if (SECTION_FG_LINE >= unsigned(action) && unsigned(action) >= SECTION_BG)
+    if (SECTION_FG_BG >= unsigned(action) && unsigned(action) >= SECTION_BG)
         ret << " " << unsigned(action) - unsigned(SECTION_BG);
     else if (action == TEXT_AREA || action == HINT_AREA || action==CURVE_TO)
         ret << " " << x1 << " " << y1 << " " << x2 << " " << y2;
@@ -118,13 +119,14 @@ const Contour &Shape::GetCover() const
  * Empty current_section_data and invalidate cover.*/
 void Shape::ProcessSubPath(bool close)
 {
-    if (current_section_data.size()) {
-        if (close && current_section_data.front().GetStart() != current_section_data.back().GetEnd())
-            current_section_data.emplace_back(current_section_point, XY(current_section_data.front().GetStart()));
-        GetSection(current_section).append(std::move(current_section_data));
-        cover_fresh = false;
-        current_section_data.clear();
-    }
+    if (current_section_data.empty()) return;
+    if (close && current_section_data.front().GetStart() != current_section_data.back().GetEnd())
+        current_section_data.emplace_back(current_section_point, XY(current_section_data.front().GetStart()));
+    if (current_section == 3 && current_section_data.IsClosed() && current_section_data.IsConnected()) 
+        bg.append(current_section_data);
+    GetSection(current_section).append(std::move(current_section_data));
+    cover_fresh = false;
+    current_section_data.clear();
 }
 
 void Shape::Add(ShapeElement &&e)
@@ -176,6 +178,10 @@ void Shape::Add(ShapeElement &&e)
         ProcessSubPath(current_section<2); //make previous section 0 and 1 closed
         current_section = 2;
         break;
+    case ShapeElement::SECTION_FG_BG:
+        ProcessSubPath(current_section < 2); //make previous section 0 and 1 closed
+        current_section = 3;
+        break;
     }
 }
 
@@ -384,13 +390,14 @@ bool ShapeCollection::Add(std::string_view shape_text, MscError &Error)
             break;
         case 'S':
             if (block) break;
-            if (shape_text[pos+1]==' ' && unsigned(shape_text[pos+2]-'0') <=2)
+            if (shape_text[pos+1]==' ' && unsigned(shape_text[pos+2]-'0') <=3)
                 switch (shape_text[pos+2]) {
                 case '0': shapes.back().Add(ShapeElement(ShapeElement::SECTION_BG, 0, 0, 0, 0, 0, 0)); break;
                 case '1': shapes.back().Add(ShapeElement(ShapeElement::SECTION_FG_FILL, 0, 0, 0, 0, 0, 0)); break;
                 case '2': shapes.back().Add(ShapeElement(ShapeElement::SECTION_FG_LINE, 0, 0, 0, 0, 0, 0)); break;
+                case '3': shapes.back().Add(ShapeElement(ShapeElement::SECTION_FG_BG, 0, 0, 0, 0, 0, 0)); break;
                 } else
-                Error.Error(fpos, "'S' should be followed by a space and an integer between 0 and 2. Ignoring this line.");
+                Error.Error(fpos, "'S' should be followed by a space and an integer between 0 and 3. Ignoring this line.");
                 break;
         case 'T':
             if (block) break;
