@@ -44,13 +44,16 @@ struct ShapeElement
         HINT_AREA = 4,
         TEXT_AREA = 5,
         SECTION_BG = 6,
-        SECTION_FG_FILL = 7,
-        SECTION_FG_LINE = 8,
-        PORT = 9
+        SECTION_FG_FILL = 7,  //SECTION_BG + 1
+        SECTION_FG_LINE = 8,  //SECTION_BG + 2 
+        SECTION_FG_BG = 9,    //SECTION_BG + 3  
+        PORT = 10,
+        _MAX = 10
     };
 
     Type action;
-    static const char act_code[];
+    static constexpr const char act_code[] = "LMCEHTSSSSP";
+    static_assert(size_t(_MAX)+2 == sizeof(act_code));
     double x, y;
     double x1, y1; //x1 is also used for compass points 
     double x2, y2;
@@ -87,7 +90,7 @@ private:
     Path fg_line;    ///<The foreground path (section 2) (will be stroked with line.color and line.type)
     mutable bool cover_fresh; ///<Is the calculated 'cover' member up-to-date?
     mutable Contour cover;    ///<The area covered by all sections combined in the coordinate space of the shape.
-    Path  &GetSection(unsigned section) { return section==0 ? bg : section==1 ? fg_fill : fg_line; }
+    Path  &GetSection(unsigned section) { return section==0 ? bg : section==1 ? fg_fill : fg_line; } ///<fg_line is also returned for section == 3
     unsigned current_section=0;///<During parsing, it stores which section are we actually gathering.
     Path current_section_data; ///<During parsing, it stores the current section so far.
     XY current_section_point;  ///<During parsing, it stores the cursor.
