@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include "chartbase.h"
 #include "cgen_progress.h"
 #include "canvas.h"
+#include "cgencommon.h"
 
 /** Jumps over tabs and spaces in 's'. If 's' starts with none of these, nothing is done.
  * Any zero character stops processing.
@@ -190,9 +191,9 @@ bool ProgressBase::DoCallback(double p) const
 
 const char LabelInfo::labelTypeChar[] = "-EADBbPVCSN";
 
-std::unique_ptr<Csh> Chart::CshFactory(Csh::FileListProc) const
+std::unique_ptr<Csh> Chart::CshFactory(Csh::FileListProc, const LanguageCollection* languages) const
 {
-    return std::make_unique<DummyCsh>();
+    return std::make_unique<DummyCsh>(languages);
 }
 
 /** Find what fill to apply in the range Y.We assume all fill blocks span the
@@ -233,11 +234,12 @@ FillAttr Chart::GetBkFill(Range Y, bool top) const noexcept {
 }
 
 
-Chart::Chart(FileReadProcedure *p, void *param) :
+Chart::Chart(FileReadProcedure *p, void *param, const LanguageCollection* languages) :
     first_include(-1),
     total(0, 0, 0, 0), copyrightTextHeight(0),
     noLabels(0), noOverflownLabels(0),
-    file_read_proc(p), file_read_proc_param(param)
+    file_read_proc(p), file_read_proc_param(param),
+    languages(languages)
 {
     ignore_designs = false;
     prepare_for_tracking = true;
@@ -278,6 +280,45 @@ Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
     return {};
 }
 
+std::unique_ptr<Chart> Chart::CompileInlineText(std::string_view language, const FileLineColRange& attr_lang,
+                                                std::string_view text, const FileLineCol& start, const Element* from,
+                                                std::string_view gui_state) {
+    std::unique_ptr<Chart> chart;
+    if (language.empty())
+        Error.Error(start, "Inlining a text of unspecified language.");
+    else if (!languages)
+        Error.Error(attr_lang.start, "You cannot inline here, languages not available.");
+    else if (auto *l = languages->GetLanguage(language)) {
+        chart = l->create_chart(file_read_proc, file_read_proc_param);
+        chart->GetProgress()->active = false;
+        chart->prepare_for_tracking = prepare_for_tracking;
+        chart->prepare_element_controls = prepare_element_controls;
+        chart->GetProgress()->callback = nullptr;
+        if (forced_pedantic)
+            chart->SetPedantic(*forced_pedantic);
+        chart->Shapes = l->shapes;
+        chart->ImportDesigns(l->designs);
+        if (forced_design.size())
+            chart->ApplyForcedDesign(forced_design);
+        chart->SetDefaultFont(forced_fontface, forced_fontlang, forced_fontlocation);
+        chart->SetPedantic(GetPedantic());
+        chart->DeserializeGUIState(gui_state);
+        //Copy the files we use so that they are the same in both Charts.
+        //So that we can use the same FileLineCol values.
+        //For this reason we skip any errors the design libs of the inlined
+        //language contained: the designlib files of the inlined lang do not show up
+        //(but the ones of 'this' do, but not referenced).
+        chart->Error.Files = Error.Files;
+        chart->inlined_from = from;
+        chart->ParseText(text, start);
+        Error.Files = chart->Error.Files; //Make sure the 'file' of all locations in 'chart' is valid in 'this'
+        chart->CompleteParse(false, false, XY(0, 0), false, /*collect links*/true);
+    } else
+        Error.Error(attr_lang.start, StrCat("Unrecognized language '", language, "'."),
+                    StrCat("Use one of ", languages->GetLanguageNames(), "."));
+    return chart;
+}
+
 
 
 /** Draws the chart into one of more files.
@@ -307,15 +348,17 @@ Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
 * @param [in] margins For fixed size pages, these are the margins
 * @param [in] ha The horizontal alignment for fix size pages (-1: left, 0: center, +1:right)
 * @param [in] va The vertical alignment for fix size pages (-1:up, 0: center, +1:bottom)
-* @param [in] generateErrors If true and we cannot avoid overfill, we generate a warning.
+* @param [in] generateErrors If true and we cannot avoid overfill or have something the output file 
+*                            cannot take (like PPT drawing inline charts with sheering), we generate a warning/error.
 *                            Errors are also generated on file creation and similar hard errors.
-* @returns False on error (but not on warning) irrespective of `generateErrors`. */
+* @returns False on file errors (but not on warnings or other drawing errors) irrespective of `generateErrors`. */
 bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
                        std::string_view fn, bool bPageBreak, bool ignore_pagebreaks,
                        const char *include_chart_text, unsigned only_page,
                        const XY &pageSize, const double margins[4],
                        int ha, int va, bool generateErrors)
 {
+
     const PBDataVector pageBreakData = GetPageVector();
     const unsigned from = only_page ? only_page : pageBreakData.size()<=1 || ignore_pagebreaks ? 0 : 1;
     const unsigned till = only_page ? only_page : pageBreakData.size()<=1 || ignore_pagebreaks ? 0 : unsigned(pageBreakData.size());
@@ -514,6 +557,42 @@ void Chart::SetToEmpty()
     used_shapes.clear();
 }
 
+ElementRef Chart::FindElement(const XY& xy) const {
+    const Area* pArea = AllCovers.InWhichFromBack(xy);
+    if (!pArea || !pArea->arc) return {};
+    if (auto* pData = pArea->arc->GetInlinedChartData(); pData && pData->chart) 
+        if (auto ret = pData->chart->FindElement(XY(xy).Apply(pData->parent_to_inline))) {
+            if (ret.inline_data)
+                ret.inline_data->AddParent(*pData);
+            else
+                ret.inline_data.emplace(pData->parent_to_inline.Invert(), pData->chart.get());
+            return ret;
+        }
+    return { pArea->arc, {} };
+}
+
+ElementRef Chart::FindElement(const FileLineCol& l) const {
+    //in the map linenum_ranges are sorted by increasing length, we search the shortest first
+    auto i = std::ranges::find_if(AllElements, [&l](auto& p) { return p.first.start <= l && l <= p.first.end; });
+    if (i == AllElements.end() || !i->second)
+        return {};
+    if (auto* pData = i->second->GetInlinedChartData(); pData && pData->chart) {
+        //Calculate the position inside the inlined text
+        _ASSERT(pData->file_pos.start.line <= l.line);
+        const size_t line = l.line - pData->file_pos.start.line + 1;
+        const FileLineCol inside_inlining(l.file, line, 1 < line ? l.col : l.col - pData->file_pos.start.col + 1);
+        if (auto ret = pData->chart->FindElement(inside_inlining)) {
+            if (ret.inline_data)
+                ret.inline_data->AddParent(*pData);
+            else
+                ret.inline_data.emplace(pData->parent_to_inline.Invert(), pData->chart.get());
+            return ret;
+        }
+    }
+    return { i->second, {} };
+}
+
+
 std::pair<const std::string*, FileLineCol>
 Chart::Include(std::string_view filename, const FileLineCol &linenum_command)
 {
@@ -564,6 +643,3 @@ bool Chart::CheckColor(const Attribute & a)
                     StringFormat::RemovePosEscapesCopy(a.value.c_str()) + "'. Ignoring attribute.");
     return false;
 }
-
- /**Call the callback if p is larger than granularity demands*/
-
