@@ -1,6 +1,6 @@
 /*
 This file is part of Msc-generator.
-Copyright (C) 2008-2023 Zoltan Turanyi
+Copyright (C) 2008-2024 Zoltan Turanyi
 Distributed under GNU Affero General Public License.
 
 Msc-generator is free software: you can redistribute it and/or modify
@@ -28,10 +28,11 @@ along with Msc-generator.  If not, see <http://www.gnu.org/licenses/>.
 #include <variant>
 #include <unordered_map>
 #include "version.h"
+#include "cgen_attribute.h"
 #include "cgencommon.h"
 
 std::string VersionText(char a=LIBMSCGEN_MAJOR, char b = LIBMSCGEN_MINOR, char c = LIBMSCGEN_SUPERMINOR);
-std::string ReadFile(FILE *in, bool till_zero=false);
+std::string ReadFile(FILE *in, bool till_zero=false, bool add_newline=true);
 std::string version(const LanguageCollection &languages);
 std::string license(const LanguageCollection &languages);
 
@@ -78,7 +79,7 @@ public:
 
 
 struct GUIInit {
-    LanguageCollection languages;
+    const LanguageCollection &languages;
     Chart::FileReadProcedure *file_read_proc = nullptr;
     void *file_read_proc_param = nullptr;
     OpenNamedFileFunction *file_open_proc = nullptr;
@@ -86,6 +87,12 @@ struct GUIInit {
     std::string file_to_open_path;
     std::string file_to_open_name;
     std::string type;
+    std::string fontface, fontlang;
+    std::map<std::string, std::vector<std::string_view>> command_line_arguments;
+    std::vector<std::pair<Attribute, std::vector<std::string_view>>> command_line_options;
+    std::optional<bool> pedantic;
+    std::string start_chart_text;
+    std::string forced_design;
     std::string *load_data = nullptr;
     double max_fps = 60;
 };
@@ -93,7 +100,7 @@ struct GUIInit {
 
 std::variant<int, GUIInit>
 do_main(const std::vector<std::string> &args,
-        LanguageCollection &&languages,
+        LanguageCollection &languages,
         std::vector<std::pair<std::string, std::string>> &&design_files,
         OpenNamedFileFunction ReadNamedFile,
         Chart::FileReadProcedure file_read_proc, void *file_read_proc_param,
