@@ -283,8 +283,8 @@ struct TRMatrix;
  */
 
 struct XY {
-    double x; ///< The x coordinate.
-    double y; ///< The y coordinate.
+    double x = 0; ///< The x coordinate.
+    double y = 0; ///< The y coordinate.
     constexpr XY &   operator +=(const XY &wh)      noexcept {x+=wh.x; y+=wh.y; return *this;}    ///< Shifts the point by `wh`.
     constexpr XY &   operator -=(const XY &wh)      noexcept {x-=wh.x; y-=wh.y; return *this;}    ///< Shifts the point by `-wh`.
     constexpr XY     operator +(const XY &wh) const noexcept {return XY(x+wh.x, y+wh.y);}         ///< Returns the point shifted by `wh`, keeping the original.
