@@ -141,8 +141,9 @@ constexpr int WhichSectionSide(const XY&A, const XY&B, const XY&P) noexcept
 struct PathPos
 {
     size_t edge;  ///<The index of the edge containing the position
-    double pos;   ///<The position whithin the edge, in between [0..1]. Note that since edges do not necessarily connect pos(e,1) may not equal pos(e+1,0) as with contours.
-    constexpr bool operator <(const PathPos &o) const noexcept { return std::tie(edge, pos) < std::tie(o.edge, o.pos); }
+    double pos;   ///<The position within the edge, in between [0..1]. Note that since edges do not necessarily connect pos(e,1) may not equal pos(e+1,0) as with contours.
+    constexpr bool operator <(const PathPos& o) const noexcept { return std::tie(edge, pos) < std::tie(o.edge, o.pos); }
+    constexpr bool operator <=(const PathPos &o) const noexcept { return operator<(o) || operator==(o); }
     /** Two PathPos are equal also if they both refer to the same vertex.
      * E.g, (edge=4, pos=1) == (edge=5, pos=0). */
     bool operator ==(const PathPos &o) const { 
@@ -167,6 +168,17 @@ struct PathPos
     }
 };
 
+/** Returns true if the order of the 3 positions is correct.
+ * We assume a non-cyclic path.*/
+inline bool between(const PathPos& a, const PathPos& b, const PathPos& c) noexcept { return a <= b && b <= c; }
+/** Returns true if the order of the 3 positions is correct.
+ * We assume a cyclic path, thus if we start from a, warp to the beginning of the path,
+ * hit b and then hit c, we are also good. Or we start from a, hit b then warp and hit c.*/
+inline bool between_warp(const PathPos& a, const PathPos& b, const PathPos& c) noexcept { 
+    return between(a, b, c) || between(b, c, a) || between(c, a, b);
+}
+
+
 
 /** Helper class to collect data during distance calculation.
  *
