@@ -1433,7 +1433,6 @@ void Edge::TangentFrom(const Edge &o, XY clockwise[2], XY cclockwise[2]) const
         }
         return;
     }
-    unsigned num_iter = 0;
     std::array<Edge, 3> e1, e2;
     double infl[2];
     unsigned num1 = 1, num2 = 1;
@@ -1475,7 +1474,6 @@ void Edge::TangentFrom(const Edge &o, XY clockwise[2], XY cclockwise[2]) const
     }
     double delta = 0;
     do {
-        num_iter++;
         XY store_cwise = clockwise[1];
         XY store_ccwise = cclockwise[1];
         for (unsigned u = 0; u<num1; u++) {
