@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -172,7 +172,7 @@ void DoubleMap<element>::Prune()
 }
 
 
-/** Contains a single, contigous non-degenerate 2D shape with no holes. Can be clockwise or counterclockwise.
+/** Contains a single, contiguous non-degenerate 2D shape with no holes. Can be clockwise or counterclockwise.
  * @ingroup contour_internal
  *
  * We maintain the following invariants in this class.
@@ -195,7 +195,7 @@ class SimpleContour
     friend class Contour;
     friend class HoledSimpleContour;
     friend class ContourList;
-    friend class ContoursHelper; //For walk related memebers
+    friend class ContoursHelper; //For walk related members
     friend struct node;
 
 public:
@@ -226,9 +226,9 @@ public:
     using MetaDataList = EdgeList<ExpandMetaData>;
     static constexpr double EXPAND_TOLERANCE = 1e-3;
 
-    SimpleContour() noexcept : clockwise_fresh(true), boundingBox_fresh(true), area_fresh(true), clockwise(true), boundingBox(false), area(0)  {}  ///<Create an empty shape with no edges.
+    SimpleContour() noexcept : boundingBox(false), area(0) {}  ///<Create an empty shape with no edges.
     SimpleContour(SimpleContour&& p) noexcept = default;  ///<Move content of another shape. The other shape gets undefined.
-    SimpleContour(const SimpleContour &p) : edges(p.edges), clockwise_fresh(p.clockwise_fresh), boundingBox_fresh(p.boundingBox_fresh), area_fresh(p.area_fresh), clockwise(p.clockwise), boundingBox(p.boundingBox), area(p.area)  {} ///<Create a copy of another shape
+    SimpleContour(const SimpleContour& p) = default; ///<Create a copy of another shape
     SimpleContour(double sx, double dx, double sy, double dy) {operator = (Block(sx,dx,sy,dy));} ///<Create a rectangle
     SimpleContour(const Block &b) {operator =(b);} ///<Create a rectangle
     SimpleContour(XY a, XY b, XY c); ///<Create an (eventually clockwise) triangle. Empty if degenerate.
@@ -248,7 +248,7 @@ public:
     const Edge &at_prev(size_t i) const noexcept { return edges.at_prev(i); } ///<Returns reference to edge before index `i`. Wraps around at first edge.
 
     void swap(SimpleContour &b) noexcept;
-    void clear() noexcept { edges.clear(); clockwise_fresh = true; area_fresh = true; boundingBox_fresh = true;  boundingBox.MakeInvalid(); area = 0.; clockwise = true; } ///<Empty the shape by deleting all edges.
+    void clear() noexcept { boundingBox.MakeInvalid(); area = 0.; edges.clear();  } ///<Empty the shape by deleting all edges.
     operator Path() const { return Path(edges); } ///<Convert our content to a Path
 
     bool AddAnEdge(const Edge &edge);
@@ -259,15 +259,15 @@ public:
      * @returns true if a change was made.*/
     bool Simplify(double tolerance = SMALL_NUM) { if (!edges.Simplify(tolerance)) return false; /*assume bb and area remained the same*/ return true; }
 
-    void Shift(const XY &xy) noexcept { boundingBox.Shift(xy); edges.Shift(xy); } ///<Translates the shape.
-    void Scale(double sc) noexcept { boundingBox.Scale(sc); edges.Scale(sc); area *= sc*sc; } ///<Scale the shape.
-    void Scale(const XY &sc) noexcept { boundingBox.Scale(sc); edges.Scale(sc); area_fresh=false; } ///<Scale the shape.
-    void SwapXY() noexcept { _ASSERT(IsSane()); boundingBox.SwapXY(); edges.SwapXY(); Invert(); area *= -1; clockwise = !clockwise; } ///<Transpose the shape: swap XY coordinates (but keep clockwisedness).
-    void Rotate(double cos, double sin) noexcept { edges.Rotate(cos, sin); boundingBox_fresh = false; } ///<Rotate the shape by `radian`. `sin` and `cos` are pre-computed values.
-    void RotateAround(const XY&c, double cos, double sin) noexcept { edges.RotateAround(c, cos, sin); boundingBox_fresh = false; } ///<Rotate the shape by `radian` around `c`. `sin` and `cos` are pre-computed values.
-    SimpleContour &NonInvertingTransform(const TRMatrix &M) noexcept { _ASSERT(!M.IsFlipping()); edges.Transform(M); area_fresh = boundingBox_fresh = false; return *this; } ///<Apply a non-inverting transformation. We retain our clockwisedness status
+    void Shift(const XY &xy) noexcept { if (!is_bb_dirty()) boundingBox.Shift(xy); edges.Shift(xy); } ///<Translates the shape.
+    void Scale(double sc) noexcept { if (!is_bb_dirty()) boundingBox.Scale(sc); edges.Scale(sc); area *= sc*sc; } ///<Scale the shape.
+    void Scale(const XY &sc) noexcept { if (!is_bb_dirty()) boundingBox.Scale(sc); edges.Scale(sc); } ///<Scale the shape.
+    void SwapXY() noexcept { _ASSERT(IsSane()); if (!is_bb_dirty()) boundingBox.SwapXY(); edges.SwapXY(); Invert(); area *= -1; } ///<Transpose the shape: swap XY coordinates (but keep clockwisedness).
+    void Rotate(double cos, double sin) noexcept { edges.Rotate(cos, sin); dirty_bb(); } ///<Rotate the shape by `radian`. `sin` and `cos` are pre-computed values.
+    void RotateAround(const XY&c, double cos, double sin) noexcept { edges.RotateAround(c, cos, sin); dirty_bb(); } ///<Rotate the shape by `radian` around `c`. `sin` and `cos` are pre-computed values.
+    SimpleContour& NonInvertingTransform(const TRMatrix& M) noexcept { _ASSERT(!M.IsFlipping()); edges.Transform(M); dirty_bb(); dirty_area(); return *this; } ///<Apply a non-inverting transformation. We retain our clockwisedness status
     SimpleContour CreateNonInvertingTransformed(const TRMatrix &M) const { SimpleContour ret(*this); return ret.NonInvertingTransform(M); } ///<Apply a non-inverting a transformation. We retain our clockwisedness status
-    SimpleContour &InvertingTransform(const TRMatrix &M) noexcept { _ASSERT(M.IsFlipping()); edges.Transform(M); Invert(); clockwise = !clockwise; area_fresh = boundingBox_fresh = false; return *this; } ///<Apply an inverting transformation. We retain our clockwisedness status
+    SimpleContour &InvertingTransform(const TRMatrix &M) noexcept { _ASSERT(M.IsFlipping()); edges.Transform(M); Invert(); dirty_bb(); if (area != infa && area != 0) area = area < 0 ? maxa : lowa; return *this; } ///<Apply an inverting transformation. We retain our clockwisedness status
     SimpleContour CreateInvertingTransformed(const TRMatrix &M) const { SimpleContour ret(*this); return ret.InvertingTransform(M); }  ///<Apply an inverting a transformation. We retain our clockwisedness status
 
     EContourRelationType RelationTo(const SimpleContour& c) const;
@@ -296,12 +296,12 @@ public:
     const Edge& back() const noexcept { return edges.back(); }   ///<Returns const reference to the last edge
     const Path &GetEdges() const & noexcept { return edges; }    ///<Returns const reference to the edges
     Path && GetEdges() && noexcept { return std::move(edges); }  ///<Moves the edges out
-    const Block &GetBoundingBox() const noexcept { if (!boundingBox_fresh) CalculateBoundingBox(); return boundingBox; } ///<Returns the bounding box.
-    bool GetClockWise() const noexcept { if (!clockwise_fresh) CalculateClockwise(); return clockwise; }              ///<Returns if the shape is clockwise.
+    const Block &GetBoundingBox() const noexcept { if (is_bb_dirty()) CalculateBoundingBox(); return boundingBox; } ///<Returns the bounding box.
+    bool GetClockWise() const noexcept { if (area==infa) CalculateClockwise(); return 0<=area; }              ///<Returns if the shape is clockwise.
     bool IsEmpty() const noexcept {return edges.size()==0;}            ///<Returns if the shape is empty (no edges).
     bool IsRectangle() const noexcept; ///<Returns true if we are equivalent to our bounding box. False for empty.
     bool IsSane() const;
-    double GetArea() const noexcept {return area_fresh ? area : CalculateArea();}  ///<Return surface area of shape.
+    double GetArea() const noexcept {return area==infa || area==lowa || area==maxa ? CalculateArea() : area;}  ///<Return surface area of shape.
     double GetCircumference(bool include_hidden=false) const noexcept;
     XY CentroidUpscaled() const noexcept;
 
@@ -358,16 +358,20 @@ protected:
     Edge       &at_next(size_t i) { return edges.at_next(i); } ///<Returns reference to edge after index `i`. Wraps around at last edge.
     Edge       &at_prev(size_t i) { return edges.at_prev(i); } ///<Returns reference to edge before index `i`. Wraps around at first edge.
 
+    void dirty_area() noexcept { if (area != infa && area != 0) area = area < 0 ? lowa : maxa; } //we invalidate the area, but keep clockwisedness info
+    void dirty_bb() noexcept { boundingBox.x.from = infa; }
+    bool is_bb_dirty() const noexcept { return boundingBox.x.from == infa; }
+
     const Block &CalculateBoundingBox() const noexcept;
     double CalculateArea() const noexcept;
-    void CalculateClockwise() const noexcept { if (!clockwise_fresh) { if (!area_fresh) CalculateArea(); clockwise = area>=0; clockwise_fresh = true; } } ///<Calcualtes whether we are clockwise or not.
+    void CalculateClockwise() const noexcept { if (area==infa) CalculateArea(); } ///<Calcualtes whether we are clockwise or not.
     bool Sanitize(double tolerance = SMALL_NUM);
 
     void assign_dont_check(std::span<const XY> v);       ///<Set shape content to `v`. Assume edges in `v` connect and do not cross.
     void assign_dont_check(std::initializer_list<XY> v); ///<Set shape content to `v`. Assume edges in `v` connect and do not cross.
     template <typename Edgeish>
-    void assign_dont_check(std::span<const Edgeish> v) { edges.assign(v.begin(), v.end()); clockwise_fresh = boundingBox_fresh = area_fresh = false; Sanitize(); }     ///<Set shape content to `v`. Assume edges in `v` connect and do not cross.
-    void assign_dont_check(Path &&v) { edges.swap(v); clockwise_fresh = boundingBox_fresh = area_fresh = false; Sanitize(); }      ///<Set shape content to `v`. Assume edges in `v` connect and do not cross.
+    void assign_dont_check(std::span<const Edgeish> v) { area = infa; dirty_bb(); edges.assign(v.begin(), v.end()); Sanitize(); } ///<Set shape content to `v`. Assume edges in `v` connect and do not cross.
+    void assign_dont_check(Path&& v) { area = infa; dirty_bb(); edges.swap(v); Sanitize(); }                                      ///<Set shape content to `v`. Assume edges in `v` connect and do not cross.
 
     EContourRelationType CheckContainmentHelper(const SimpleContour& b, size_t threshold=1, size_t allow=0) const;
     EContourRelationType CheckContainmentForOperation(const SimpleContour& b) const;
@@ -381,13 +385,21 @@ protected:
                         int last_type, int stype) const;
 
 private:
-    Path edges; ///<An ordered list of (directed, connected) edges.
-    mutable bool clockwise_fresh : 1 = false;   ///<If true, the member 'clockwise' is up-to-date
-    mutable bool boundingBox_fresh : 1 = false; ///<If true, the member 'boundingBox' is up-to-date
-    mutable bool area_fresh : 1 = false;        ///<If true, the member 'area' is up-to-date
-    mutable bool clockwise : 1;         ///<Tells if the simplecontour is clockwise or counterclockwise
-    mutable Block  boundingBox;         ///<The bounding box for the shape.
-    mutable double area;                ///<The surface area of the shape. Negative for counterclockwise shapes.
+    /** Instead of extra bits, we use the value of the 'area' to determine clockwiseness. 
+     * 0: The contour has no area. We treat these as clockwise by definition.
+     * 0<: The contour has positive area, clockwise
+     * <0: The contour has negative area, counterclockwise
+     * max(): We do not know the area, but the contour is clockwise
+     * lowest(): We do not know the area, but the contour is counterclockwise
+     * infinity: We do not know the area nor the clockwiseness. */
+    static constexpr double infa = std::numeric_limits<double>::infinity();
+    static constexpr double lowa = std::numeric_limits<double>::lowest();
+    static constexpr double maxa = std::numeric_limits<double>::max();
+    static_assert(std::numeric_limits<double>::has_infinity, "We need infinity");
+
+    mutable Block boundingBox;   ///<The bounding box for the shape. If 'dirty' (from.x==infa), we have not yet calculated it. If 'invalid' (either for x or y from=maxa&till=lowa), we have no edges.
+    mutable double area = infa;  ///<The surface area of the shape. Negative for counterclockwise shapes. See the special values above.
+    Path edges;                  ///<An ordered list of (directed, connected) edges.
 };
 
 template <typename Edgeish>
@@ -445,6 +457,30 @@ inline double SimpleContour::OffsetBelow(const SimpleContour &below, double &tou
     return do_offsetbelow(below, touchpoint, offset);
 }
 
+/** Calculate & cache the surface area of the shape.
+ *
+ * This is based on <http://www.mathopenref.com/coordpolygonarea.html>
+ * For each edge, we calculate the area above it (the area between the edge and Y axis,
+ * but since y grows downward, the area is "above" the edge).
+ * The value is signed. If the edge is from right to left it is positive, else negative.
+ * Since (for straight edges) the area would contain "start.x*start.y-end.x*end.y",
+ * which will cancel out. Plus it is easier to return twice the area.
+ * Thus we request edge to return
+ * "2*(area_above_edge - start.x*start.y + end.x*end.y)", this is what
+ * Edge::GetAreaAboveAdjusted() returns.
+ *
+ * After this function the area cache will hold this value for instant access.
+ *
+ * @returns the calculated area. Negative if shape is counterclockwise.
+ */
+inline double SimpleContour::CalculateArea() const noexcept
+{
+    double ret = 0;
+    for (const auto &e: edges)
+        ret += e.GetAreaAboveAdjusted();    
+    return area = ret/2;
+}
+
 
 
 } //namespace contour
