@@ -84,13 +84,13 @@ public:
         {return area;};
     /** An area to highlight when the mouse moves over the Element*/
     const Contour &GetAreaToDraw() const
-        {return draw_is_different ? area_draw : area;}
+        {return draw_is_different ? area_draw : static_cast<const Contour&>(area);}
     /** An area which shall not get covered by notes, if possible. */
     const Contour &GetAreaImportant() const
         {return area_important;}
     /** An area to the edge of which notes made to this element will point to*/
     const Contour &GetAreaToNote() const
-        {return area_to_note.IsEmpty() ? area : area_to_note;}
+        {return area_to_note.IsEmpty() ? static_cast<const Contour&>(area) : area_to_note;}
     /** An second try for an area to the edge of which notes with an 'at' clause will point to*/
     const Contour &GetAreaToNote2() const {return area_to_note2;}
     /** Return the list of GUI controls associated with this Element*/
