@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -134,8 +134,8 @@ class FileLineCol
     const FileLineCol *Down(unsigned l) const noexcept { return l ? next->Down(l-1) : this; }
 public:
     int file = -1;   ///<The number of the file. Names are stored in MscError::Files. -1 if invalid location.
-    size_t line = 0; ///<The line of the location
-    size_t col = 0;  ///<The character inside the line. First char is index zero.
+    size_t line = 0; ///<The (1-based) line of the location 
+    size_t col = 0;  ///<The (1-based) character inside the line
     EInclusionReason reason = EInclusionReason::TOP_LEVEL; ///<Why are we included
     constexpr FileLineCol() noexcept = default;
     constexpr FileLineCol(int a, size_t b, size_t c=0, EInclusionReason r= EInclusionReason::TOP_LEVEL) noexcept
@@ -379,8 +379,8 @@ public:
     /** Clearing all collected errors and warnings, but keeps the files.*/
     void Clear() noexcept {Messages.clear(); hadFatal=warn_mscgen=false; }
     /** Copies both files and messages from another Error collection.
-     * It is best if files do not overlap. Useful only when merging errors
-     * from different file parses.*/
+     * It asserts that our 'Files' member starts with all the files in the
+     * object we merge in.*/
     MscError &operator+=(const MscError &);
 
     void RemovePathFromErrorMessages();
