@@ -81,7 +81,7 @@ struct SourceLocationRange : std::conditional<CSH, CshPos, LineColRange>::type {
             os << '(' << loc.first_pos.char_index;
             if (loc.first_pos.char_index != loc.first_pos.byte_index)
                 os << "/" << loc.first_pos.byte_index;
-            os << "->" << loc.end_pos;
+            os << "->" << loc.end_pos.char_index;
             if (loc.end_pos.char_index != loc.end_pos.byte_index)
                 os << "/" << loc.end_pos.byte_index;
             return os << ')';
