@@ -3,11 +3,18 @@
 #undef _CRT_SECURE_NO_WARNINGS
 #include <unordered_map>
 #include <unordered_set>
+#include <filesystem>
+#include <cstdlib>
 #include "ppt.h"
 #include "ppt_def.h"
 #include "cgen_shapes.h"
 #include "commandline.h"
 #include "tinyxml2.h"
+#include "error.h"
+#include "utf8utils.h"
+#ifndef WIN32
+#include <unistd.h>
+#endif
 
 /** A note on points: When exporting to PPT, one chart pixel is emitted as one PPT point (1/72 of an inch).
  * PPT pixels (1/96 of an inch) are never used.*/
@@ -17,42 +24,49 @@
 //We do escaping
 //We could use XML escaping for <>&"' but since we need to correctly encode whitespace anyway,
 //we use a single system of escapes with '~'
-//~:~~, \n:~n, \t:~t, N consecutive space (between 2-9):~N, <:~l, >:~g, ":~q, ':~a, &:~@
-//supply the field name and the value. We always end in "~|"
+//~:~~, \n:~n, \t:~t, N consecutive space (between 0-9):~N, <:~l, >:~g, ":~q, ':~a, &:~@
+//Note that ~0 simply means nothing (zero spaces), it can be used to break up a character
+//sequence in the input that for some reason is not accepted by PowerPoint
+//(Version 2208 Build 16.0.15601.20540) 64-bit) (but is accepted by Word).
+//Currently known such character input strings are: "cid".
+//Supply the field name and the value. We always end in "~|"
 static void escape_embedded_alt_text(std::string& ret, std::string_view t, std::string_view v) {
     _ASSERT(std::ranges::all_of(t, [](char c) {return isalnum(c)|| c=='_'; }));
     using namespace std::string_view_literals;
     ret.reserve(ret.size()+v.size()*5/4+t.size()+1);
     ret.append(t).push_back('=');
     unsigned spaces = 0;
-    for (auto c: v)
-        if (c==' ')
+    for (; v.size(); v.remove_prefix(1))
+        if (const char c = v.front(); c==' ')
             switch (++spaces) {
-            case 1: ret += ' '; break;
-            case 2: ret.back() = '~'; ret.push_back('2'); break;
+            case 1: ret += ' '; continue;
+            case 2: ret.back() = '~'; ret.push_back('2'); continue;
             case 9: spaces = 0; FALLTHROUGH; //end this run
-            default: ++ret.back(); break; //increase digit after '~'
-            } else {
-                spaces = 0;
-                if (c=='~')
-                    ret += "~~"sv;
-                else if (c=='\n')
-                    ret += "~n"sv;
-                else if (c=='\t')
-                    ret += "~t"sv;
-                else if (c == '<')
-                    ret += "~l";
-                else if (c == '>')
-                    ret += "~g";
-                else if (c == '"')
-                    ret += "~q";
-                else if (c == '&')
-                    ret += "~@";
-                else if (c == '\'')
-                    ret += "~a";
-                else
-                    ret += c;
+            default: ++ret.back(); continue; //increase digit after '~'
             }
+        else {
+            spaces = 0;
+            if (c == '~')
+                ret += "~~"sv;
+            else if (c == '\n')
+                ret += "~n"sv;
+            else if (c == '\t')
+                ret += "~t"sv;
+            else if (c == '<')
+                ret += "~l";
+            else if (c == '>')
+                ret += "~g";
+            else if (c == '"')
+                ret += "~q";
+            else if (c == '&')
+                ret += "~@";
+            else if (c == '\'')
+                ret += "~a";
+            else if (v.starts_with("cid"))
+                ret += "c~0"; //break up "cid" - MS PPT does not accept it
+            else
+                ret += c;
+        }
     ret += "~|";
 }
 
@@ -81,6 +95,8 @@ static std::pair<std::string_view, std::string> descape_embedded_alt_text_field(
             case 'q': ret.second += '"'; break;
             case 'a': ret.second += '\''; break;
             case '@': ret.second += '&'; break;
+            case '0': break;
+            case '1': ret.second += ' '; break;
             case '2':
             case '3':
             case '4':
@@ -158,15 +174,35 @@ std::string EmbedChartData::Escape() const {
 }
 
 std::string my_tmpnam() {
-    std::string ret(L_tmpnam, '\0');
-    tmpnam(ret.data());
-    ret.resize(strlen(ret.c_str()));
-    return ret;
+#if WIN32
+    WCHAR szTempFileName[MAX_PATH];
+    WCHAR lpTempPathBuffer[MAX_PATH];
+    // Gets the temp path env string (no guarantee it's a valid path).
+    if (const DWORD dwRetVal = GetTempPathW(MAX_PATH, lpTempPathBuffer);
+             dwRetVal > MAX_PATH || (dwRetVal == 0))
+        throw std::runtime_error("Cannot obtain temp dir name.");
+    // Generates a temporary file name.
+    if (!GetTempFileNameW(lpTempPathBuffer, // directory for tmp files
+                          L"mscgen",        // temp file name prefix
+                          0,                // create unique name - checks if unique and creates zero-length file
+                          szTempFileName))  // buffer for name
+        throw std::runtime_error("Cannot obtain temp file name.");
+    return ConvertFromUTF16_to_UTF8(szTempFileName);
+#else
+    namespace fs = std::filesystem;
+    std::string ret = fs::temp_directory_path() / "msc-gen.XXXXXX";
+    if (auto fd = mkstemp(ret.data()); fd == -1)
+        throw std::runtime_error("Cannot create temp file: " + my_strerror(fd));
+    else {
+        close(fd);
+        return ret;
+    }
+#endif
 }
 
 /************************************************************************************/
 
-static constexpr const char xml_content_types_preamble[] = 
+static constexpr const char xml_content_types_preamble[] =
 R"(<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <Types xmlns="http://schemas.openxmlformats.org/package/2006/content-types">)"
     R"(<Default Extension="jpeg" ContentType="image/jpeg"/>)"
@@ -196,7 +232,7 @@ static constexpr const char xml_content_types_postscript[] =
     R"(<Override PartName="/ppt/theme/theme1.xml" ContentType="application/vnd.openxmlformats-officedocument.theme+xml"/>)"
 R"(</Types>)";
 
-static constexpr const char xml_app_template[] = 
+static constexpr const char xml_app_template[] =
 R"(<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <Properties xmlns="http://schemas.openxmlformats.org/officeDocument/2006/extended-properties" xmlns:vt="http://schemas.openxmlformats.org/officeDocument/2006/docPropsVTypes">)"
 	//R"(<TotalTime>0</TotalTime>)"
@@ -245,7 +281,7 @@ R"(<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 	//R"(<LinksUpToDate>false</LinksUpToDate>)"
 	//R"(<SharedDoc>false</SharedDoc>)"
 	//R"(<HyperlinksChanged>false</HyperlinksChanged>)"
-	//R"(<AppVersion>%d.%d.%d</AppVersion>)" //Better not add current version for reproducible production of pptxes across versions.    
+	//R"(<AppVersion>%d.%d.%d</AppVersion>)" //Better not add current version for reproducible production of pptxes across versions.
 R"(</Properties>)";
 
 
@@ -385,14 +421,14 @@ R"(<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
     "<p:nvPr/>"
             "</p:nvGrpSpPr>"
             "<p:grpSpPr/>";
-static constexpr const char xml_slide_postscript[] = 
+static constexpr const char xml_slide_postscript[] =
 "</p:spTree></p:cSld></p:sld>";
 
 static constexpr const char xml_slide_rels_preamble[] =
 R"(<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships">)"
 R"(<Relationship Id="rId1" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/slideLayout" Target="../slideLayouts/slideLayout1.xml"/>)";
-static constexpr const char xml_slide_rels_postscript[] = 
+static constexpr const char xml_slide_rels_postscript[] =
 R"(</Relationships>)";
 
 //returns compound type and dash type
@@ -528,7 +564,7 @@ auto fill_attrs(const FillAttr& f, bool force_white=false) {
         gslist = gs(0., c1) + gslist + gs(1., c2);
         break;
     case EIntGradType::BUTTON:
-        gslist = gs(0, c1.Lighter(0.8)) 
+        gslist = gs(0, c1.Lighter(0.8))
                + gs(0.43, c1)
                + gs(0.5, c1.Darker(0.1))
                + gs(1, c1);
@@ -539,17 +575,17 @@ auto fill_attrs(const FillAttr& f, bool force_white=false) {
         "<a:lin ang=\"" + std::to_string(unsigned(60000*angle)) + "\" scaled=\"1\"/>"
         "</a:gradFill>";
 }
-auto preset_geom(std::string_view type, std::string_view adjustments = {}) { 
+auto preset_geom(std::string_view type, std::string_view adjustments = {}) {
     if (adjustments.empty())
         return "<a:prstGeom prst=\"" + std::string(type) + "\"/>";
     return StrCat("<a:prstGeom prst=\"", type, "\">",
                   "<a:avLst>", adjustments, "</a:avLst>" "</a:prstGeom>");
 }
 
-std::pair<std::string, Block> path_list(const Path &P, bool closed) { 
+std::pair<std::string, Block> path_list(const Path &P, bool closed) {
     Block rect(false);
     std::string path;
-    for (auto& i: P) 
+    for (auto& i: P)
         if (i.IsDot()) continue;
         else if (i.IsStraight())
             (rect += i.GetStart()) += i.GetEnd();
@@ -587,24 +623,22 @@ std::pair<std::string, Block> path_list(const Path &P, bool closed) {
 
 std::string label_font_attrs(const StringFormat& sf, double scale_font = 1, unsigned hyperlink_id = 0) {
     const double size = sf.fontType != EFontType::MSC_FONT_SMALL ? *sf.normalFontSize : *sf.smallFontSize; //superscript & subscript automatically reduce font size
-    const unsigned scaled_size = unsigned(size * scale_font * 100); // Thanks, M$, that all else is in EMUs but this one is in (hundreths of) _points_. 
-    std::string ret(2000, 0);
+    const unsigned scaled_size = unsigned(size * scale_font * 100); // Thanks, M$, that all else is in EMUs but this one is in (hundreths of) _points_.
     std::string lang = sf.lang->size()!=2 ? "en-en" : *sf.lang+"-"+*sf.lang; //if not a two letter ISO code, we use english
-    int s = snprintf(ret.data(), ret.size(), R"(<a:rPr dirty="0" sz="%u" err="1" lang="%s")", scaled_size, lang.c_str());
-    _ASSERT(0<s && (unsigned)s<ret.size()-1);
-    ret.resize(s);
+    auto ret = R"(<a:rPr dirty="0" sz=")" + std::to_string(scaled_size) + R"(" err="1" lang=")" + lang + '"';
     if (sf.italics.value_or(ETriState::no) == ETriState::yes) ret += R"( i="1")";
     if (sf.bold.value_or(ETriState::no) == ETriState::yes) ret += R"( b="1")";
     if (sf.underline.value_or(ETriState::no) == ETriState::yes) ret += R"( u="sng")";
     if (sf.fontType == EFontType::MSC_FONT_SUBSCRIPT) ret += R"( baseline="-40000")";
     if (sf.fontType == EFontType::MSC_FONT_SUPERSCRIPT) ret += R"( baseline="30000")";
     ret += '>';
-    if (sf.color && sf.color!=ColorType::black()) ret += solid_fill(*sf.color); // MS: Color must come before typeface 
+    if (do_not_draw_fonts) ret += solid_fill(ColorType(0,0,0,0));
+    else if (sf.color && sf.color != ColorType::black()) ret += solid_fill(*sf.color); // MS: Color must come before typeface
     if (sf.bgcolor && !sf.bgcolor->IsFullyTransparent()) ret += "<a:highlight>" + color(*sf.bgcolor) + "</a:highlight>";
     if (sf.face && sf.face->size()) ret += "<a:latin typeface=\"" + *sf.face + "\"/>";  // TODO non-latin?
-    else ret += R"(<a:latin typeface="Arial"/>)"; 
+    else ret += R"(<a:latin typeface="Arial"/>)";
     if (hyperlink_id) ret.append(R"(<a:hlinkClick r:id="rId)").append(std::to_string(hyperlink_id)).append(
-        R"(">)" //here comes a MS specific extension to reflect the color of the formatting 
+        R"(">)" //here comes a MS specific extension to reflect the color of the formatting
 		    R"(<a:extLst>)"
                 R"(<a:ext uri="{A12FA001-AC4F-418D-AE19-62706E023703}">)"
                     R"(<ahyp:hlinkClr xmlns:ahyp="http://schemas.microsoft.com/office/drawing/2018/hyperlinkcolor" val="tx"/>)"
@@ -620,10 +654,10 @@ std::string label_para_attrs(const StringFormat& sf) {
     switch (sf.ident.value_or(EIdentType::MSC_IDENT_CENTER)) {
     default:
     case EIdentType::MSC_IDENT_INVALID:
-    case EIdentType::MSC_IDENT_LEFT: break;   
+    case EIdentType::MSC_IDENT_LEFT: break;
     case EIdentType::MSC_IDENT_CENTER: ret += R"(algn="ctr" )"; break;
     case EIdentType::MSC_IDENT_RIGHT: ret += R"(algn="r" )"; break;
-    }    
+    }
     ret += R"(marL="0" marR="0" latinLnBrk="0"/>)"; //latinLnBrk="0" pevents moving a whole word to the next line
     return ret;
 }
@@ -670,7 +704,7 @@ std::string PptFile::MakeLabel(const Label& L, std::string_view asterisk_replace
         const double line_scale_font = scale_font* sqrt(l.GetScale().x * l.GetScale().y);
         std::string replaceto;
         while (text.length() && text[0]) {
-            size_t length;            
+            size_t length;
             switch (sf.ProcessEscape(text, length, false, true, &replaceto)) {
             case StringFormat::NON_FORMATTING:
                 check_start_r(line_scale_font);
@@ -722,8 +756,8 @@ unsigned PptFile::GetLinkId(std::string_view url) {
         rel_content += R"(" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink" Target=")";
         rel_content += url;
         rel_content += R"(" TargetMode="External"/>)";
-        return id; 
-    } 
+        return id;
+    }
     return i-links.begin()+2;
 }
 
@@ -744,7 +778,7 @@ std::string PptFile::MakeElement(const Block& rect, const LineAttr& line, ArrowH
         R"(<a:off x=")" + std::to_string(xywh.x) + R"(" y=")" + std::to_string(xywh.y) + R"("/>)"
         R"(<a:ext cx=")" + std::to_string(xywh.w) + R"(" cy=")" + std::to_string(xywh.h) + R"("/>)"
         "</a:xfrm>"
-        + props 
+        + props
         + line_attrs(line, a_start, a_end) +
         "</p:spPr>"
         "<p:style>"
@@ -767,7 +801,7 @@ void PptFile::reset() {
     clip = Block(false);
 }
 
-bool PptFile::new_file(std::string_view fname, int pageno, const XY& page_size_in_points_) {
+bool PptFile::new_file(std::string_view fname, int pageno [[maybe_unused]], const XY& page_size_in_points_) {
     if (file_name.size()) {
         err = {};
         return false;
@@ -796,7 +830,7 @@ bool PptFile::new_file(std::string_view fname, int pageno, const XY& page_size_i
 }
 
 bool PptFile::set_page_size(const XY &origSize, const XY &origOffset,
-                            const XY &headingLeftingSize, const XY &user_scale, 
+                            const XY &headingLeftingSize, const XY &user_scale,
                             int h_alignment, int v_alignment, const Block& raw_page_clip,
                             double copyrightTextHeight) {
     if (origSize.x<1 || origSize.y<1) return false;
@@ -962,13 +996,13 @@ bool PptFile::add(GenericShape&& shape) {
         if (p->path.size()==1 && p->path.front().IsStraight()) {
             const XY& s = p->path.front().GetStart();
             const Block bb(s, p->path.front().GetEnd());
-            content += MakeElement(bb, p->line, p->aStart, p->aEnd, 
+            content += MakeElement(bb, p->line, p->aStart, p->aEnd,
                                    "p:cxnSp", "p:nvCxnSpPr", "p:cNvCxnSpPr", preset_geom("line"), {},  //Lines are expressed as a line from the upper left of the bounding box
                                    s==bb.UpperLeft() ? "" : s==bb.UpperRight() ? " flipH=\"1\"" :      //to the lower right, with some flip transformations applied.
                                    s==bb.LowerLeft() ? " flipV=\"1\"" : " flipV=\"1\" flipH=\"1\"");
         } else {
             auto [geom, rect] = path_list(p->path, false);
-            content += MakeElement(rect, p->line, p->aStart, p->aEnd, "p:sp", "p:nvSpPr", "p:cNvSpPr",     
+            content += MakeElement(rect, p->line, p->aStart, p->aEnd, "p:sp", "p:nvSpPr", "p:cNvSpPr",
                                    "<a:custGeom>" + std::move(geom) + "</a:custGeom>" "<a:noFill/>");
         }
     } else if (auto* const b = dynamic_cast<GSBox*>(&shape)) { //GSBox and GSShape
@@ -1037,7 +1071,7 @@ bool PptFile::add(GenericShape&& shape) {
                     break;
                 }
                 //for text-only shapes, expand the bounding box, to avoid accidental text wrapping
-                if (b->line.IsFullyTransparent() && b->fill.IsFullyTransparent() && b->shadow.IsEmpty() && !b->label.IsEmpty())
+                if (b->line.IsFullyTransparent() && b->fill.IsFullyTransparent() && b->shadow.IsNone() && !b->label.IsEmpty())
                     bb.Expand(1);
             }
             const std::string_view valign = b->valign==ETextAlign::Min ? "anchor=\"t\" " : b->valign==ETextAlign::Mid ? "anchor=\"ctr\" " : "anchor=\"b\" ";
@@ -1163,7 +1197,7 @@ std::string PptFile::append_page(ZipArchive archive, unsigned* page) {
     const unsigned id = std::ranges::max_element(slides, {}, &Slide::id)->id+1;
     unsigned rId = 0; //count how many relationships we have
     for (tinyxml2::XMLElement* child = elems[1]->FirstChildElement(); child; child = child->NextSiblingElement())
-        rId++; 
+        rId++;
     std::string rId_str, fn_str;
     do {
         rId++;
@@ -1175,16 +1209,16 @@ std::string PptFile::append_page(ZipArchive archive, unsigned* page) {
         fn_str = "slides/slide"+std::to_string(fn)+".xml";
     } while (std::ranges::find(slides, fn_str, &Slide::filename) != slides.end());
     //Add page to presentation.xml
-    elems[0] = elems[0]->InsertNewChildElement("p:sldId"); 
+    elems[0] = elems[0]->InsertNewChildElement("p:sldId");
     elems[0]->SetAttribute("id", id);
     elems[0]->SetAttribute("r:id", rId_str.c_str());
-    
-    //Add page to presentation.xml.rels 
+
+    //Add page to presentation.xml.rels
     elems[1] = elems[1]->InsertNewChildElement("Relationship");
     elems[1]->SetAttribute("Id", rId_str.c_str());
     elems[1]->SetAttribute("Type", "http://schemas.openxmlformats.org/officeDocument/2006/relationships/slide");
     elems[1]->SetAttribute("Target", fn_str.c_str());
-    
+
     //Add page to content
     elems[2] = elems[2]->InsertNewChildElement("Override");
     elems[2]->SetAttribute("PartName", ("/ppt/"+fn_str).c_str());
@@ -1197,9 +1231,9 @@ std::string PptFile::append_page(ZipArchive archive, unsigned* page) {
         if (files[u].content.ends_with('\0')) files[u].content.pop_back(); //TinyXML includes the terminating zero in CStrSize()
     }
 
-    //Now add an empty slide 
+    //Now add an empty slide
     files.emplace_back("ppt/"+fn_str, StrCat(xml_slide_preamble, xml_slide_postscript));
-    files.emplace_back("ppt/slides/_rels/"+fn_str.substr(7)+".rels", StrCat(xml_slide_rels_preamble, xml_slide_rels_postscript));  
+    files.emplace_back("ppt/slides/_rels/"+fn_str.substr(7)+".rels", StrCat(xml_slide_rels_preamble, xml_slide_rels_postscript));
 
     if (page) *page = slides.size()+1;
     return AddToZip(archive, files);
@@ -1215,7 +1249,7 @@ static std::optional<PptFile::ChartInPPT> ExtractChart(tinyxml2::XMLElement* pic
     if (s.id==MAX) return {};
     const char* const descr = prop->Attribute("descr");
     if (!descr) return {};
-    if (EmbedChartData chart = EmbedChartData::ParseEscapedText(descr); !chart.empty()) 
+    if (EmbedChartData chart = EmbedChartData::ParseEscapedText(descr); !chart.empty())
         s.data = std::move(chart);
     else return {};
 
@@ -1234,7 +1268,17 @@ static std::optional<PptFile::ChartInPPT> ExtractChart(tinyxml2::XMLElement* pic
     if (unsigned y = ext->UnsignedAttribute("cy", MAX); y==MAX) return {};
     else s.h = y;
 
-    prop = get_path(pic, { clipboard ? "a:blipFill": "p:blipFill", "a:blip" });
+    tinyxml2::XMLElement* blipfill = pic->FirstChildElement(clipboard ? "a:blipFill" : "p:blipFill");
+    if (!blipfill) return {};
+    if (tinyxml2::XMLElement* srcRect = blipfill->FirstChildElement("a:srcRect")) {
+        s.crop_l = srcRect->IntAttribute("l", 0);
+        s.crop_r = srcRect->IntAttribute("r", 0);
+        s.crop_t = srcRect->IntAttribute("t", 0);
+        s.crop_b = srcRect->IntAttribute("b", 0);
+    } else
+        s.crop_l = s.crop_r = s.crop_t = s.crop_b = 0;
+
+    prop = blipfill->FirstChildElement("a:blip");
     if (!prop) return {};
     const char* const rId = prop->Attribute("r:embed");
     if (!rId) return {};
@@ -1272,10 +1316,12 @@ std::string UpdateChart(const PptFile::ChartInPPT& current, const PptFile::Chart
         if (png_inner_fn.starts_with(".."))
             png_inner_fn.erase(0, 3);
     }
-    double zoom = sqrt((current.w/current.data.chart_size.x) * current.h/current.data.chart_size.y); //avg of the existing zoom in X and Y dir
+    const XY current_visible = current.crop(current.data.chart_size);
+    const XY c_visible = current.crop(c.data.chart_size);
+    double zoom = sqrt((current.w/ current_visible.x) * current.h/ current_visible.y); //avg of the existing zoom in X and Y dir
     if (slide_size)
-        zoom = std::min({ zoom, slide_size->first/c.data.chart_size.x, slide_size->second/c.data.chart_size.y }); //maximize zoom to fit
-    GenericShape::int_xy cs = { int(c.data.chart_size.x*zoom), int(c.data.chart_size.y*zoom) };
+        zoom = std::min({ zoom, slide_size->first/c_visible.x, slide_size->second/c_visible.y }); //maximize zoom to fit
+    GenericShape::int_xy cs = { int(c_visible.x*zoom), int(c_visible.y*zoom) };
     //Keep the chart center the same
     GenericShape::int_xy cp = {
         std::max(0, current.x - cs.x/2 + (int)current.w/2),
@@ -1286,7 +1332,7 @@ std::string UpdateChart(const PptFile::ChartInPPT& current, const PptFile::Chart
         if (slide_size->first<cp.y+cs.y) cp.y = 0;
     }
 
-    //Now replace the fields need replacing: name, descr, bounding box. Id, rId remains
+    //Now replace the fields need replacing: name, descr, bounding box. Id, rId, srcRect remains
     tinyxml2::XMLElement* prop = get_path(pic, is_clipboard ? std::initializer_list{ "a:nvPicPr", "a:cNvPr" } : std::initializer_list{ "p:nvPicPr", "p:cNvPr" });
     if (!prop) return {};
     prop->SetAttribute("name", ("Msc-generator "+c.data.chart_type).c_str());
@@ -1320,7 +1366,7 @@ std::string UpdateChart(const PptFile::ChartInPPT& current, const PptFile::Chart
                 if (!choff) return {};
                 tinyxml2::XMLElement* chext = xfrm->FirstChildElement("a:chExt");
                 if (!chext) return {};
-                
+
                 if (unsigned cx = chext->UnsignedAttribute("cx"), cy = chext->UnsignedAttribute("cy")
                         ; cx ==pic_ext->UnsignedAttribute("cx")
                         && cy==pic_ext->UnsignedAttribute("cy")
@@ -1367,7 +1413,7 @@ static std::string ExtractTitle(tinyxml2::XMLElement* spTree) {
         if (!type) continue;
         using namespace std::string_view_literals;
         if (type!="title"sv && type!="ctrTitle"sv) continue;
-        
+
         //cycle through the a:r elements (for all fragments) of the first paragraph
         for (tinyxml2::XMLElement* r = get_path(sp, { "p:txBody", "a:p", "a:r" }); r; r = r->NextSiblingElement("a:r"))
             if (tinyxml2::XMLElement* t = r->FirstChildElement("a:t"))
@@ -1386,7 +1432,7 @@ std::string Print(const tinyxml2::XMLDocument& doc) {
     return ret;
 }
 
-/** Adds the PNG file type tp [Content Types].xml 
+/** Adds the PNG file type tp [Content Types].xml
  * @param file_content the content of the [Content Types].xml file. We update it if needed
  * @returns An error message on a problem, an empty string if success and file_content
  *   remained unchanged (PNG type is already added), an empty optional if PNG type added.*/
@@ -1410,7 +1456,7 @@ std::optional<std::string> AddPNGFileTypeTo(std::string& file_content) {
     return {};
 }
 
-//returns new rId and inner filename (not from the root of the zip, but starting with 'media/'). 
+//returns new rId and inner filename (not from the root of the zip, but starting with 'media/').
 //Also adds the new ref to the rels file ('Relations')
 //On error we return an empty string and the error (and make no change to 'Relations')
 std::pair<std::string, std::string> alloc_new_image(ZipArchive archive, tinyxml2::XMLElement *Relations) {
@@ -1433,7 +1479,7 @@ std::pair<std::string, std::string> alloc_new_image(ZipArchive archive, tinyxml2
     std::vector<FileInZip> dir;
     if (std::string err = GetFilesInZip(archive, dir); err.size())
         return { "", "Error in "+archive.qfname_or_clipboard()+"': I cant read the directory." };
-    std::erase_if(dir, [](const FileInZip& f) { 
+    std::erase_if(dir, [](const FileInZip& f) {
         //drop first dir, it can be 'ppt' or 'clipboard'
         const size_t dash = f.inner_filename.find('/');
         if (dash==std::string::npos) return true;
@@ -1451,7 +1497,7 @@ std::pair<std::string, std::string> alloc_new_image(ZipArchive archive, tinyxml2
     Relations->SetAttribute("Id", rId_str.c_str());
     Relations->SetAttribute("Type", "http://schemas.openxmlformats.org/officeDocument/2006/relationships/image");
     Relations->SetAttribute("Target", ("../"+png_inner_fn).c_str());
-    
+
     return { std::move(rId_str), std::move(png_inner_fn) };
 }
 
@@ -1487,7 +1533,7 @@ std::string PptFile::add_images(ZipArchive archive, const std::vector<ChartToPPT
     files_in_zip.emplace_back("[Content_Types].xml", "");
     std::string err = ExtractFromZip(archive, files_in_zip);
     if (files_in_zip.size()<slides_index.size()*2+1)
-        return "Error in "+archive.qfname_or_clipboard()+"': I dont find a slide, its rels file or '[Content_Types].xml'.";
+        return "Error in "+archive.qfname_or_clipboard()+"': I don't find a slide, its rels file or '[Content_Types].xml'.";
 
     //Start by adding PNG type to content types, if needed
     if (auto oErr = AddPNGFileTypeTo(files_in_zip.back().content); oErr && oErr->length())
@@ -1599,12 +1645,12 @@ std::string PptFile::add_images(ZipArchive archive, const std::vector<ChartToPPT
             unsigned ch = c->chart;
             std::optional<ChartInPPT> oChart;
             tinyxml2::XMLElement* pic;
-            std::unordered_set<std::string> rIds; //The rId of the other charts 
+            std::unordered_set<std::string> rIds; //The rId of the other charts
             for (pic = spTree->FirstChildElement("p:pic"); pic; pic = pic->NextSiblingElement("p:pic")) {
                 oChart = ExtractChart(pic, rel, false);
                 if (!oChart) continue;
-                if (--ch) rIds.insert(oChart->rId); 
-                else break;                
+                if (--ch) rIds.insert(oChart->rId);
+                else break;
             }
             if (pic==nullptr)
                 return "Page "+std::to_string(c->slide)+" of "+archive.qfname_or_clipboard()+"' contains only "+std::to_string(c->chart-ch)+
@@ -1669,7 +1715,7 @@ std::string PptFile::parse_charts(ZipArchive archive, std::vector<ChartInPPT>& c
             return "Error in "+archive.qfname_or_clipboard()+" page "+std::to_string(page+1)+":"+files[slides.size()+page].inner_filename+"': Cant find Relationships.";
 
         unsigned count = 0;
-        //Cylce through all the "p:pic" elements and check if they match 
+        //Cylce through all the "p:pic" elements and check if they match
         for (tinyxml2::XMLElement* pic = spTree->FirstChildElement("p:pic"); pic; pic = pic->NextSiblingElement("p:pic"))
             if (auto oChart = ExtractChart(pic, Relationships, false)) {
                 oChart->slide = page+1;
@@ -1737,7 +1783,7 @@ std::string PptClipboard::update(const std::vector<PptFile::ChartToPPT>& C, Open
         unsigned ch = c.chart;
         std::optional<PptFile::ChartInPPT> oChart;
         tinyxml2::XMLElement* pic;
-        std::unordered_set<std::string> rIds; //The rId of the other charts 
+        std::unordered_set<std::string> rIds; //The rId of the other charts
         for (pic = lockedCanvas->FirstChildElement("a:pic"); pic; pic = pic->NextSiblingElement("a:pic")) {
             oChart = ExtractChart(pic, Relationships, true);
             if (oChart && --ch==0) break;
@@ -1814,16 +1860,16 @@ std::string PptClipboard::create(EmbedChartData&& data, std::string&& png_conten
     std::string descr = data.Escape();
     std::string drawing1xml(strlen(xml_clipboard_template)+descr.length()+255, '\0');
     const XY slide_size = GetPhysicalPageSize(EPageSize::ON_SCREEN_4_3_L);
-    const double zoom = data.chart_size.x>0 && data.chart_size.y>0 
-        ? std::min({ 1., slide_size.x/data.chart_size.x, slide_size.y/data.chart_size.y }) 
+    const double zoom = data.chart_size.x>0 && data.chart_size.y>0
+        ? std::min({ 1., slide_size.x/data.chart_size.x, slide_size.y/data.chart_size.y })
         : 1.;
     const auto [X,Y]= emu(data.chart_size*zoom);
     const int len = snprintf(drawing1xml.data(), drawing1xml.size(), xml_clipboard_template,
                              X, Y, X, Y, data.chart_type.c_str(), descr.c_str(), X, Y);
     if (len<1) return {};
     drawing1xml.resize(len);
-   
-    std::vector<FileInZip> files = { 
+
+    std::vector<FileInZip> files = {
         {"clipboard/drawings/drawing1.xml", std::move(drawing1xml)},
         {"clipboard/media/image1.png", std::move(png_content)}
     };
