@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -161,6 +161,12 @@ bool SimpleStyle::AddAttribute(const Attribute &a, Chart*chart)
             if (f_fill)
                 return fill.AddAttribute(a, chart, type);
             return false;
+        case EColorMeaning::LINE_FILL:
+            if (f_line)
+                num += line.AddAttribute(a, chart, type);
+            if (f_fill)
+                num += fill.AddAttribute(a, chart, type);
+            return num != 0;
         default:
             _ASSERT(0);
             FALLTHROUGH;
@@ -243,6 +249,8 @@ bool SimpleStyle::DoIAcceptUnqualifiedColorAttr() const
         return f_text || f_line;
     case EColorMeaning::LINE_VLINE_TEXT:
         return f_text || f_line;
+    case EColorMeaning::LINE_FILL:
+        return f_fill || f_line;
     default:
         _ASSERT(0);
         FALLTHROUGH;
