@@ -3,6 +3,7 @@
 
 #include <unordered_set>
 #include "commandline.h"
+#include "clipboard.h"
 
 //Seamless conversion between contour::XY and ImGui's ImVec2
 #define IM_VEC2_CLASS_EXTRA                                                              \
@@ -54,23 +55,6 @@ void GUIBeep();
 std::vector<std::string> FileListProc(std::string_view prefix_, std::string_view included_from_);
 std::optional<std::string> ShowDocumentation(const std::string& doc_name);///< Open the named doc, or return an error string
 std::optional<std::string> ShowLink(std::string_view url); ///<Open the web browser with this URL (in UTF-8)
-/** The formats we are interested in on the clipboard. */
-enum class ClipboardFormat {
-    Art_GVML,    ///<Zipped XML containing something copied from Office
-    Escaped_Text ///<Plain text starting with "Msc-generator~|"
-};
-/** Returns the content of the clipboard (if interesting for us).
- * (2 formats are interesting, see 'ClipboardFormat'.) Any value
- * is only returned once, that is, after we have returned the content,
- * subsequent calls to this function will return the empty string
- * unless new data of interesting type is copied to the clipboard.*/
-std::pair<ClipboardFormat, std::string> GetClipboard();
-/** Place this to the clipboard.
- * @params [in] format What is the format of 'content' we shall use when placin to clipboard.
- * @params [in] content The content to place to the clipboard.
- * @params [in] type For Art_GVML, this contains the human readable chart type. Ignored for Escaped_Text.
- * @returns true if the operation failed and re-trying it makes sense. (E.g., clipboard was locked, etc.) */
-bool CopyToClipboard(ClipboardFormat format, const std::string& content, std::string_view type = {});
 
 struct GUIReturn {
     enum Action {
