@@ -120,9 +120,6 @@ GraphChart::GraphChart(FileReadProcedure *p, void *param, const LanguageCollecti
 GraphChart::~GraphChart()
 {
     orig_lang_clear();
-    //delete remnants of potentially broken parsing
-    for (auto p : parse_stack)
-        delete p;
 }
 
 
@@ -604,9 +601,6 @@ void GraphChart::Draw(Canvas & c, bool pageBreaks, unsigned page)
 void GraphChart::SetToEmpty()
 {
     orig_lang_clear();
-    //delete remnants of potentially broken parsing
-    for (auto p : parse_stack)
-        delete p;
     Graphs.clear();
     ChartBase<GraphContext>::SetToEmpty();
     pageBreakData.emplace_back(XY(0, 0), XY(0, 0));
