@@ -89,7 +89,7 @@ protected:
 public:
     /** @name variables used during parsing
     * @{ */
-    std::list<SubGraph*> parse_stack;      ///<The subgraphs we are in the middle of defining. Empty if we fill the main graph (just for parsing)
+	NPtrList<SubGraph> parse_stack;        ///<The subgraphs we are in the middle of defining. Empty if we fill the main graph (just for parsing)
     std::string        last_subgraphname;  ///<The name of the lastly opened subgraph (just for parsing)
     FileLineColRange   last_subgraphspos;  ///<The position of the lastly opened subgraph hdr in the file. Invalid if none (just for parsing)
     bool               lst_scope_is_graph; ///<The last opened scope is a top-level graph scope and not a subgraph (just for parsing)
