@@ -107,7 +107,7 @@ struct ArrowSegmentData {
 struct ArcTypePlusDir
 {
     ArrowSegmentData arc; ///<What kind of an arrow (or box) are we, and where are we lost (if at all)
-EDirType dir;         ///<Are we forward, backward or bidir or without arrowhead.
+    EDirType dir;         ///<Are we forward, backward or bidir or without arrowhead.
     ArcTypePlusDir() = default;
     ArcTypePlusDir(EArcSymbol t, EArrowLost l, EDirType d, const FileLineColRange &p = FileLineColRange()) :
         arc(t, l, p), dir(d) {}
@@ -298,7 +298,9 @@ public:
     virtual bool CanBeNoted() const {return false;}
     /** True if the entity is likely to produce height and can be the target of a vertical or a byref symbol. */
     virtual bool CanBeAlignedTo() const { return false; }
-    /** True if this entity aligns to another entity by referring to it. 
+    /* True if the entity has a centerline to which markers can be aligned.*/
+    virtual bool HasCenterline() const { return false; }
+    /** True if this entity aligns to another entity by referring to it.
      * Such entities are laid out independently. */
     virtual bool IsByRef() const { return false; }
     /** For by ref symbols and verticals, it returns the element to
@@ -402,7 +404,8 @@ public:
         LEFT_SIDE,  ///<At the phyiscal left side (can be source or dest, if those are meaningful)
         RIGHT_SIDE  ///<At the phyiscal right side (can be source or dest, if those are meaningful)
     };
-    /** Get the y position of the end, for the purposes of joining to it.
+    /** Get the y position of the end, for the purposes of joining to it or to align a centerline.
+    * Elements that CanJoin() or HasCenterline() shall support this.
     * Shall be called only after Layout()*/
     virtual double GetCenterline(CenterlineSelector where [[maybe_unused]]) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
     /** Get the activation status of the entity at the dst end of the arrow,
@@ -488,6 +491,7 @@ public:
     /** Creates an indicator object at Entity `s`*/
     Indicator(MscChart *chart, EntityRef s, const MscStyleCoW &st, const FileLineColRange &l);
     bool CanBeAlignedTo() const override { return true; }
+    bool HasCenterline() const override { return true; }
     /** True if `src` and `dst` are properly set.*/
     bool IsComplete() const;
     /** Set which entities the iterator is between. */
@@ -614,7 +618,7 @@ struct VertXPos
     static bool IsTwoEntityPos(EPosType t) { return t==POS_CENTER; }
 
     bool valid = false;          ///<True if the position is valid
-    EPosType pos;                ///<The type of the position
+    EPosType pos{};              ///<The type of the position
     EntityRef entity1 = nullptr; ///<The first (or only one) entity
     EntityRef entity2 = nullptr; ///<The second entity, used only for POS_CENTER
     FileLineColRange e1line;     ///<Location of the mention of the first entity in the input line (inside this vertpos fragment)
@@ -728,6 +732,7 @@ public:
     /** Standard constructor. 'ys' shall be the height of the drawn arrow. */
     SelfArrow(EArcSymbol t, std::string_view s, const FileLineColRange &sl,
                  MscChart *msc, double ys);
+    bool HasCenterline() const override { return true; }
     Arrow *AddSegment(ArrowSegmentData data, std::string_view m, const FileLineColRange &ml,
                       const FileLineColRange &l) override;
     Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
@@ -813,6 +818,7 @@ public:
              MscChart *msc, bool fw);
     /** Constructor used while converting a collapsed Box to an BlockArrow*/
     DirArrow(const NEntityList &el, bool bidir, EArcSymbol boxsymbol, const LabelledArc &al);
+    bool HasCenterline() const override { return true; }
     Arrow *AddSegment(ArrowSegmentData data, std::string_view m, const FileLineColRange &ml,
                       const FileLineColRange &l) override;
     Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
