@@ -137,6 +137,7 @@ public:
     /** Create a series using a box as a first element (and taking ownership!)*/
     BoxSeries(gsl::owner<Box*> first, gsl::owner<AttributeList*> l);
     bool CanBeAlignedTo() const override { return true; }
+    bool HasCenterline() const override { return true; }
     bool StoreNoteTemporarily(Note* cn) override;
     void ReinsertTmpStoredNotes(ArcList& list, ArcList::iterator after) override;
     /** Expands the file_pos of the first element in a series (used when adding the 'box' keyword)*/
@@ -238,7 +239,10 @@ public:
     /** Add actual content */
     PipeSeries* AddArcList(gsl::owner<ArcList*> l);
     bool CanBeAlignedTo() const override { return true; }
+    bool HasCenterline() const override { return true; }
+
     EDirType GetToucedEntities(NEntityList &el) const override;
+    double GetCenterline(CenterlineSelector /*where*/) const override { return yPos + total_height / 2; }
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
         Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     void FinalizeLabels(Canvas &canvas) override;
@@ -270,18 +274,20 @@ protected:
     bool wide;                 ///<If true, we keep no margin and add no gap above & below (for copyright text)
     const double extra_space;  ///<Extra vertical space above and below: depends on how we created this object
 
-    mutable double centerline; ///<Y coordinate of the centerline
+    mutable double centerline; ///<Y coordinate of the centerline relative to yPos
     mutable double text_margin;///<left and right text margin from chart edge
     mutable double line_margin;///<left and right line margin from chart edge
     mutable Contour text_cover;///<The cover of the label
 public:
     Divider(EArcSymbol t, MscChart *msc);
     bool CanBeAlignedTo() const override { return !nudge; } //nudges cannot be targets of a vertical
+    bool HasCenterline() const override { return !nudge; } //nedges have no centerline
     bool BeforeAutoGenEntities() const override { return title; }
     void SetStyleBeforeAttributes(AttributeList *l) override;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh, bool nudge, bool title);
     static bool AttributeValues(std::string_view attr, Csh &csh, bool nudge, bool title);
+    double GetCenterline(CenterlineSelector /*where*/) const override { return yPos + centerline; }
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
