@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 #include <map>
 #include <set>
 #include <vector>
-#include <stack>
+#include <unordered_map>
 #include "chartbase.h"
 #include "mscstyle.h"
 #include "mscentity.h"
@@ -179,7 +179,7 @@ class DistanceRequirementsSection
     const MscElement * const   element; ///<If we are not aligned to a marker but to an element, this is the one.
     const bool                 top;     ///<If we are aligned to an element, this tells if we are aligned to its top or bottom.
     std::map<unsigned, double> left;    ///<Space requirements on the left side of entities
-    std::map<unsigned, double> right;   ///<Space requirements on the left side of entities
+    std::map<unsigned, double> right;   ///<Space requirements on the right side of entities
     std::map<IPair, double>    pairs;   ///<Space requirements between arbitrary (potentially non-neighbouring) pairs of entities
     unsigned                   left_entity;  ///<The index of the leftmost entity touched
     unsigned                   right_entity; ///<The index of the rightmost entity touched
@@ -205,8 +205,8 @@ public:
      * Returns true, if an increase has been made. */
     /** Indicate that a certain entity is touched */
     void InsertEntity(unsigned e) { left_entity = std::min(left_entity, e); right_entity = std::max(right_entity, e); }
-    unsigned QueryLeftEntity() const { return left_entity; } ///<Returns the leftmost touched entity, nullptr if none
-    unsigned QueryRightEntity() const { return right_entity; } ///<Returns the righmost touched entity, nullptr if none
+    unsigned QueryLeftEntity() const { return left_entity; } ///<Returns the leftmost touched entity, NO_LEFT_ENTITY if none
+    unsigned QueryRightEntity() const { return right_entity; } ///<Returns the righmost touched entity, NO_RIGHT_ENTITY if none
     DistanceRequirementsSection &operator +=(const DistanceRequirementsSection &d);
     void CombineLeftRightToPair_Add(double gap);
 };
@@ -376,8 +376,7 @@ struct LayoutColumn
 /** Data we store about a named arc, that can be referenced via @\r(<name>) */
 struct MscRefNameData : public RefNameData
 {
-    ArcBase    *arc;         ///<The arc or nullptr if not shown (set in FinalizeLabels())
-    MscRefNameData() : arc(nullptr) {}
+    ArcBase* arc = nullptr;         ///<The arc or nullptr if not shown (set in FinalizeLabels())
 };
 
 class MscProgress : public Progress<EBulkSection, EBulkSection::MAX_BULK_SECTION,
@@ -434,7 +433,7 @@ public:
     std::string GetLanguageDefaultText() const override; ///<Returns the default text of the chart in UTF-8
     bool GetLanguageHasAutoheading() const override { return true; } ///<True if the chart has automatic headings
     bool GetLanguageHasElementControls() const override { return true; } ///<True if the elements of the chart support GUI controls
-    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const override;
+    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, const LanguageCollection* languages) const override;
     std::map<std::string, std::string> RegisterLibraries() const override;
     /** @} */
 
@@ -460,9 +459,15 @@ public:
     std::map<string, MarkerData, std::less<>> Markers; ///<A set of named markers. Each one was defined via a 'mark' command.
     std::map<double, FillAttr>    Background;      ///<A map of background fill indexed by the position downward of which this fill is applicable. Each entry was created by a background.* chart option.
     Contour                       HideELinesHere;  ///<A complex contour used as a mask when drawing entity lines.
-    ArcList                       EndNotes;        ///<We move all endnotes here during PostParseProcessArcList(). We reappend them in PostParseProcess().
+    ArcList                       EndNotes;        ///<We move all endnotes here during PostParseProcessArcList(). We re-append them in PostParseProcess().
     CommandNoteList               Notes;           ///<All notes are moved here after PostParseProcess
     NPtrList<const MscElement>    NoteBlockers;    ///<Ptr to all elements that may block a floating note and which therefore should not overlap with them (we do not own them here)
+    /** Maps elements to all the 'by ref' symbols and verticals that reference them.
+     * Filled in during Width calculation (when all elements that disappear are already
+     * removed) and used in Layout. If one element is referred to by multiple 
+     * symbols/verticals, those are stored in order of their appearance in the source files, 
+     * since unhinted 'insert' and 'emplace' always inserts at the upper bound. */
+    std::multimap<ArcBase*, ArcBase*> ByRefs;
     /** @} */
 
 protected:
@@ -523,16 +528,18 @@ public:
     EntityCollapseCatalog force_entity_collapse;      ///<These entities must be collapsed/expanded
     BoxSignatureCatalog   force_box_collapse;         ///<These boxes must be collapsed/expanded, also contains an update after parsing
     BoxSignatureCatalog   force_box_collapse_instead; ///<During parsing, this holds an updated version of the box collapse/expanded instructions (removing nonexistent boxes, etc)
+    InlineSignatureCatalog inline_gui_state;          ///<The GUI state of inlined charts
+    InlineSignatureCatalog inline_gui_state_instead;  ///<During parsing, this holds an updated version of the GUI state of inlined charts
     /** @} */
 
-    MscChart(FileReadProcedure *p, void *param);
+    MscChart(FileReadProcedure *p, void *param, const LanguageCollection* languages);
     ~MscChart() override;
-    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param)
-        { return std::make_unique<MscChart>(p, param); }
+    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param, const LanguageCollection* languages)
+        { return std::make_unique<MscChart>(p, param, languages); }
     bool AddCommandLineArg(const std::string& arg) override;
     void AddCommandLineOption(const Attribute &a) override;
     bool GetPedantic() const noexcept override { return pedantic; }
-    void SetPedantic(bool pedantic) noexcept override { this->pedantic = pedantic; }
+    void SetPedantic(bool pedantic) noexcept override { this->pedantic = pedantic; Chart::SetPedantic(pedantic); }
 
     bool ApplyForcedDesign(const string& name) override;
     bool DeserializeGUIState(std::string_view) override;
@@ -594,12 +601,12 @@ public:
     std::unique_ptr<ArcBase> PopContext();
 
     /** Parse a piece of input text as a new input file.*/
-    unsigned ParseText(std::string_view input, std::string_view filename) override;
+    void ParseText(std::string_view input, const FileLineCol& first_char) override;
     void PostParseProcessArcList(Canvas &canvas, bool hide, ArcList &arcs, bool resetiterators, EntityRef &left,
                                  EntityRef &right, Numbering &number, MscElement **note_target);
     void PostParseProcess(Canvas &canvas);
     /** Call FinalizeLabels() for each element in `arcs*/
-    template <typename list> void FinalizeLabelsArcList(list &arcs, Canvas &canvas);
+    void FinalizeLabelsArcList(ArcList &arcs, Canvas &canvas);
 
     EDirType GetTouchedEntitiesArcList(const ArcList &, NEntityList &el, EDirType dir=EDirType::INDETERMINATE) const;
 
@@ -664,19 +671,11 @@ public:
     void DrawChart(Canvas &canvas, Range yDrawing, bool pageBreaks);
 
     void DrawHeaderFooter(Canvas &canvas, unsigned page) override;
+    void Draw(Canvas& canvas, bool pageBreaks, unsigned page) override;
     void DrawComplete(Canvas &canvas, bool pageBreaks, unsigned page) override;
     void SetToEmpty() override;
 };
 
-template <typename list>
-void MscChart::FinalizeLabelsArcList(list &arcs, Canvas &canvas)
-{
-	for (auto &i : arcs) {
-		i->FinalizeLabels(canvas);
-		Progress.DoneItem(EArcSection::FINALIZE_LABELS, i->myProgressCategory);
-	}
-}
-
 } //namespace
 
 using msc::MscChart;
