@@ -474,7 +474,7 @@ void MscCsh::AddDesignLineBeginToHints()
     static const char * const design_keyword_names[] = {"invalid", nullptr,
         "defstyle", "Define a new style or change existing styles (applicable only in this design).",
         "defcolor", "Define or redefine a color (applicable only in this design).",
-        "defproc", "Define a new procedure that can be re-played later (after applying this design)."
+        "defproc", "Define a new procedure that can be re-played later (after applying this design).",
         "include", "Include the text of another file in parsing.",
         ""
     };
