@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -32,13 +32,13 @@ namespace xxx {
 class XxxCsh : public Csh
 {
 public:
-    explicit XxxCsh(Csh::FileListProc proc);
+    explicit XxxCsh(Csh::FileListProc proc, const LanguageCollection* languages);
     ~XxxCsh() override = default;
     std::unique_ptr<Csh> Clone() const override { return std::make_unique<XxxCsh>(*this); }
 
     // Inherited via Csh
     void FillNamesHints() override;
-    void ParseText(std::string&& input, int cursor_p, bool pedantic) override;
+    void ParseText(std::string&& input, CharByteIndex first_char, CharByteIndex cursor_p, bool pedantic) override;
 
 };
 
