@@ -5,17 +5,18 @@
 /** The formats we are interested in on the clipboard. */
 enum MacClipboardFormat {
     Art_GVML,    ///<Zipped XML containing something copied from Office
-    Escaped_Text ///<Plain text starting with "Msc-generator~|"
+    Escaped_Text,///<Plain text starting with "Msc-generator~|"
+    Else,        ///<Some other format
+    No_Change    ///<The clipboard has not changed
 };
 
 /** Returns the content of the clipboard (if interesting for us).
  * (2 formats are interesting, see 'MacClipboardFormat'.) Any value
  * is only returned once, that is, after we have returned the content,
- * subsequent calls to this function will return the empty string
- * unless new data of interesting type is copied to the clipboard.*/
+ * subsequent calls to this function will return No_Change.*/
 extern "C" char* MacGetClipboard(MacClipboardFormat *format, unsigned *len);
 /** Place this to the clipboard.
- * @params [in] format What is the format of 'content' we shall use when placin to clipboard.
+ * @params [in] format What is the format of 'content' we shall use when placing to clipboard.
  * @params [in] content The content to place to the clipboard. If it is text, it must be null-terminated.
  * @params [in] content_len the length of the content (not counting terminating zero for text)
  * @params [in] type For Art_GVML, this contains the human readable chart type null-terminated. Ignored for Escaped_Text.
