@@ -1,17 +1,14 @@
 /*takes one argument and generates designs.c from an incoming original_designlinb.c*/
-
 #include <stdio.h>
 
 int main(int argc, char* argv[])
 {
     if (argc<1) return 1;
-    
     printf("char %s[]=\n\"", argv[1]);
-    while (!feof(stdin)) {
-        const char c=getchar();
+    int c;
+    while ((c = getchar()) != EOF) {
         switch (c) {
         case '\n': printf("\\n\"\n\"");
-        case -1:
         case '\r': break;  /*so DOS files are handled too*/
         case '\\':
         case '\'': /*escape " and ' and \ */
@@ -22,5 +19,3 @@ int main(int argc, char* argv[])
     printf("\";\n");
     return 0;
 }
-
-
