@@ -56,7 +56,7 @@ lt_symlist_t lt_preloaded_symbols[] = {{"gvplugin_canvas_LTX_library", &gvplugin
 GVC_t *Gvc = gvContextPlugins(lt_preloaded_symbols, 1);
 //gvFreeContext(Gvc); should be called at some exit functions, but we do not do it.
 
-extern int yydebug;
+extern int block_compile_debug, block_csh_debug, flow_compile_debug, flow_csh_debug, graph_compile_debug, graph_csh_debug, msc_compile_debug, msc_csh_debug;
 
 bool progress(const ProgressBase *progress, void *, ProgressBase::EPreferredAbortMethod)
 {
@@ -243,6 +243,9 @@ std::optional<std::string> ShowLink(std::string_view url) {
 
 int main(int argc, char* argv[])
 {
+#if YYDEBUG
+    block_compile_debug = block_csh_debug = flow_compile_debug = flow_csh_debug = graph_compile_debug = graph_csh_debug = msc_compile_debug = msc_csh_debug = 1;
+#endif
     //Get msc-genrc dir
     const char *ch = getenv("MSC_GEN_RC");
     std::string hdir = ch ? ch : "";
