@@ -886,7 +886,7 @@ void contour_unit_tests(bool do_image) {
         //   9-----8,5
         Path p = {XY(10,0), XY(20,10), XY(15,10), XY(15, 15), XY(20,10), XY(15,10), XY(15, 15), XY(5, 25), XY(10,0)};
         Contour c(p);
-        Contour ref(XY(20, 10), XY(5, 25), XY(10, 0));
+        Contour ref(XY(5, 25), XY(10, 0), XY(20, 10));
         BAssert(R, c == ref, "Untangle with rightmost CP having 2 incoming edges - 1");
     }
 
@@ -914,7 +914,9 @@ void contour_unit_tests(bool do_image) {
     {
         //Edge #9 of A and Edge #7 of B almost overlap, but are further than 1e-5, which is the cutoff
         //to merge crosspoints. Their one end (endpoint of A's #9 and startpoint of B's #7) are detected as a
-        //crosspoint (because of A's #10 and B's #6), but the other end is not. This sends walking haywire.
+        //crosspoint (because of A's #10 and B's #6), but the other end is not. This sent walking haywire.
+        //Then we applied a fix, where we fixed the order of the 2 rays corresponding to these edges in the
+        //crosspoint we detected by walking better.
         Contour A = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
             Edge(XY(0x1.14f0d5866936fp+7,0x1.fafaeb6b74637p+5), XY(0x1.1212fcf57c93cp+7,0x1.f241a5bc7c809p+5)),
             Edge(XY(0x1.1212fcf57c93cp+7,0x1.f241a5bc7c809p+5), XY(0x1.0ec7782864c5dp+7,0x1.e9184e502cdbbp+5)),
@@ -969,6 +971,27 @@ void contour_unit_tests(bool do_image) {
         UnionAssert(R, A, B, "Union of two contours with missing crosspoint");
     }
 
+    {
+        Contour A = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.9407cb4936012p+5,0x1.9f013bb268577p+5), XY(0x1.a3780b24a4a0ap+5,-0x1.6p+3)),
+            Edge(XY(0x1.a3780b24a4a0ap+5,-0x1.6p+3), XY(0x1.94147ae147ae2p+5,-0x1.6p+3)),
+            Edge(XY(0x1.94147ae147ae2p+5,-0x1.6p+3), XY(0x1.94147ae147ae2p+5,-0x1.ap+3)),
+            Edge(XY(0x1.94147ae147ae2p+5,-0x1.ap+3), XY(0x1.a3f5c28f5c28fp+5,-0x1.9fffffffffffcp+3)),
+            Edge(XY(0x1.a3f5c28f5c28fp+5,-0x1.9fffffffffffcp+3), XY(0x1.b3fe0a3ba27ccp+5,0x1.9f801036523aep+5)),
+            Edge(XY(0x1.b3fe0a3ba27ccp+5,0x1.9f801036523aep+5), XY(0x1.b4p+5,0x1.ap+5), XY(0x1.b3ff56f1d0ffdp+5,0x1.9faa4c8d83f7p+5), XY(0x1.b4p+5,0x1.9fd51288ad535p+5)),
+            Edge(XY(0x1.b4p+5,0x1.ap+5), XY(0x1.a4p+5,0x1.bp+5), XY(0x1.b4p+5,0x1.a8d62888a7876p+5), XY(0x1.acd62888a7875p+5,0x1.bp+5)),
+            Edge(XY(0x1.a4p+5,0x1.bp+5), XY(0x1.94p+5,0x1.ap+5), XY(0x1.9b29d7775878bp+5,0x1.bp+5), XY(0x1.94p+5,0x1.a8d62888a7876p+5)),
+            Edge(XY(0x1.94p+5,0x1.ap+5), XY(0x1.9407cb4936012p+5,0x1.9f013bb268577p+5)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        Contour B = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.940000055f56ap+5,0x1.9fff2bed74781p+5), XY(0x1.94147ae147ae2p+5,-0x1.ap+3)),
+            Edge(XY(0x1.94147ae147ae2p+5,-0x1.ap+3), XY(0x1.a3f5c28f5c28fp+5,-0x1.ap+3)),
+            Edge(XY(0x1.a3f5c28f5c28fp+5,-0x1.ap+3), XY(0x1.940000055f56ap+5,0x1.9fff2bed74781p+5)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+
+        UnionAssert(R, A, B, "Union of two contours dunno what's wrong");
+    }
+
     {
         Contour Res = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
             Edge(XY(0x1.830dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.aaec1d5734babp+7)),
